/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * Data and methods associated with the current selection.
 */
this.Component_Selection_Model = function( component, opts ) {

	var self  = component;
	var Model = Backbone.Model.extend( {

		selected_product_data: false,

		initialize: function() {

			var selected_product      = '',
				selected_product_data = false;

			if ( self.component_options_model.available_options_data.length > 0 ) {
				$.each( self.component_options_model.available_options_data, function( index, option_data ) {
					if ( option_data.is_selected ) {
						selected_product      = option_data.option_id;
						selected_product_data = $.extend( true, {}, option_data.option_product_data );
						return false;
					}
				} );
			}

			var params = {
				selected_product:        selected_product,
				selected_variation:      '',
				selected_variation_data: false,
				selected_quantity:       0,
				selected_addons:         false,
				// NYP identified by price.
				selected_nyp:            0.0,
			};

			this.selected_product_data = selected_product_data;

			this.set( params );

			/**
			 * Update WC variations model if scenarios changed.
			 */
			composite.actions.add_action( 'component_options_state_changed_' + self.step_id, this.update_active_variations_data, 0, this );
		},

		get_product_data: function() {
			return this.selected_product_data;
		},

		get_product_image_data: function() {
			return this.selected_product_data.image_data || false;
		},

		get_variation_data: function() {
			return this.get( 'selected_variation_data' );
		},

		get_variation_image_data: function() {

			var variation_id = this.get( 'selected_variation' ),
				variations   = this.get_available_variations_data(),
				vi_data      = false;

			if ( variation_id > 0 && variations ) {
				$.each( variations, function( index, variation ) {
					if ( parseInt( variation.variation_id, 10 ) === parseInt( variation_id, 10 ) ) {
						if ( variation.image ) {
							vi_data = {
								image_src:    variation.image.src,
								image_srcset: variation.image.srcset,
								image_sizes:  variation.image.sizes,
								image_title:  variation.image.title
							};
						} else if ( variation.image_src ) {
							vi_data = {
								image_src:    variation.image_src,
								image_srcset: variation.image_srcset,
								image_sizes:  variation.image_sizes,
								image_title:  variation.image_title
							};
						}
						return false;
					}
				} );
			}

			return vi_data;
		},

		get_meta_data: function() {

			var variation_id = this.get( 'selected_variation' ),
				meta         = [];

			if ( variation_id > 0 ) {
				meta = this.get_variation_data().meta_data || [];
			}

			return meta;
		},

		get_available_variations_data: function() {
			return this.selected_product_data.variations_data || [];
		},

		get_active_variations_data: function() {
			return this.selected_product_data.active_variations_data || [];
		},

		get_type: function() {
			return this.selected_product_data.product_type || 'none';
		},

		get_price: function() {
			return this.selected_product_data.price ? Number( this.selected_product_data.price ) : 0.0;
		},

		set_price: function( value ) {
			this.selected_product_data.price = value;
		},

		get_regular_price: function() {
			return this.selected_product_data.regular_price ? Number( this.selected_product_data.regular_price ) : 0.0;
		},

		set_regular_price: function( value ) {
			this.selected_product_data.regular_price = value;
		},

		get_addons_price: function() {
			return this.selected_product_data.addons_price ? Number( this.selected_product_data.addons_price ) : 0.0;
		},

		set_addons_price: function( value ) {
			this.selected_product_data.addons_price = value;
		},

		get_tax_ratios: function() {
			return this.selected_product_data.tax_ratios || false;
		},

		set_tax_ratios: function( value ) {
			this.selected_product_data.tax_ratios = value;
		},

		get_details_html: function() {
			return this.selected_product_data.details_html || '';
		},

		get_stock_status: function() {
			return this.selected_product_data.stock_status || '';
		},

		set_stock_status: function( value ) {
			this.selected_product_data.stock_status = value;
		},

		load_selection_data: function( product_id, update_selection ) {

			update_selection = update_selection || false;

			var model = this,
				data  = {
					action:        'woocommerce_show_composited_product',
					product_id:    product_id,
					component_id:  self.component_id,
					composite_id:  composite.composite_id
				};

			// Get component selection details via ajax.
			$.ajax( {

				type:     'POST',
				url:      composite.get_ajax_url( data.action ),
				data:     data,
				timeout:  15000,
				dataType: 'json',

				success: function( response ) {

					// Cache result.
					if ( 'success' === response.result ) {

						composite.console_log( 'debug:models', '\nFetched \'Component_Selection_Model\' data: "' + self.get_title() + '", Product ID: #' + ( product_id === '' ? '0' : product_id ) );

						self.component_options_model.set_option_data( product_id, response.product_data, 'option_product_data' );

						model.trigger( 'selected_product_data_loaded', product_id, response.product_data );

						if ( update_selection ) {
							model.update_selected_product( product_id, response.product_data );
						}

					} else {

						model.trigger( 'selected_product_data_load_error', product_id );
					}

				},

				error: function() {

					model.trigger( 'selected_product_data_load_error', product_id );
				}

			} );
		},

		update_selection: function( product_id ) {

			if ( '' === product_id ) {
				this.update_selected_product( '', wc_composite_params.empty_product_data );
				return;
			}

			var product_data = self.component_options_model.get_option_data( product_id, 'option_product_data' );

			if ( product_data ) {

				this.update_selected_product( product_id, product_data );

			} else {

				composite.console_log( 'debug:models', '\nFetching \'Component_Selection_Model\' data: "' + self.get_title() + '", Product ID: #' + ( product_id === '' ? '0' : product_id ) );

				this.load_selection_data( product_id, true );
			}
		},

		update_selected_product: function( selected_product, selected_product_data ) {

			this.selected_product_data = $.extend( true, {}, selected_product_data );

			if ( this.get( 'selected_product' ) !== selected_product ) {

				var selected_quantity = 0;

				if ( selected_product ) {
					selected_quantity = 'yes' === composite.settings.component_qty_restore ? this.get( 'selected_quantity' ) : 1;
				}

				composite.console_log( 'debug:models', '\nUpdating \'Component_Selection_Model\': "' + self.get_title() + '", Attribute: "selected_product": #' + ( selected_product === '' ? '0' : selected_product ) );

				composite.debug_indent_incr();
				this.set( {
					selected_product:        selected_product,
					selected_variation:      '',
					selected_variation_data: false,
					selected_quantity:       selected_quantity,
					selected_addons:         false,
					selected_nyp:            0.0
				} );
				composite.debug_indent_decr();

				this.trigger( 'selected_product_updated' );
			}
		},

		update_selected_variation: function( selected_variation, selected_variation_data ) {

			if ( this.get( 'selected_variation' ) !== selected_variation || ! _.isEqual( this.get( 'selected_variation_data' ), selected_variation_data ) ) {

				composite.console_log( 'debug:models', '\nUpdating \'Component_Selection_Model\': "' + self.get_title() + '", Attribute: "selected_variation": #' + ( selected_variation === '' ? '0' : selected_variation ) );

				composite.debug_indent_incr();
				this.set( {
					selected_variation:      selected_variation,
					selected_variation_data: selected_variation_data
				} );
				composite.debug_indent_decr();

				this.trigger( 'selected_variation_updated' );
			}
		},

		update_selected_quantity: function( selected_qty ) {

			if ( this.get( 'selected_quantity' ) !== selected_qty ) {

				composite.console_log( 'debug:models', '\nUpdating \'Component_Selection_Model\': "' + self.get_title() + '", Attribute: "selected_quantity": ' + selected_qty );

				composite.debug_indent_incr();
				this.set( { selected_quantity: selected_qty } );
				composite.debug_indent_decr();
			}
		},

		update_selected_addons: function( selected_addons_data, selected_addons_price ) {

			if ( ! selected_addons_price ) {
				selected_addons_price = 0;
			}

			this.set_addons_price( selected_addons_price );

			if ( this.get( 'selected_addons' ) !== selected_addons_data ) {

				composite.console_log( 'debug:models', '\nUpdating \'Component_Selection_Model\': "' + self.get_title() + '", Attribute: "selected_addons"...' );

				composite.debug_indent_incr();
				this.set( { selected_addons: selected_addons_data } );
				composite.debug_indent_decr();
			}
		},

		update_nyp: function( nyp_price ) {

			if ( this.get( 'selected_nyp' ) !== nyp_price ) {

				composite.console_log( 'debug:models', '\nUpdating \'Component_Selection_Model\': "' + self.get_title() + '", Attribute: "nyp_price": ' + nyp_price );

				this.set_price( nyp_price );
				this.set_regular_price( nyp_price );

				composite.debug_indent_incr();
				this.set( { selected_nyp: nyp_price } );
				composite.debug_indent_decr();
			}
		},

		update_active_variations_data: function() {

			if ( 'variable' === this.get_type() ) {

				composite.console_log( 'debug:models', '\nUpdating "' + self.get_title() + '" variations data...' );

				/*
				 * Update WC variations model.
				 */
				var selected_variation     = self.get_selected_variation( false ),
					options_state          = self.component_options_model.get( 'options_state' ),
					variation_data         = this.get_available_variations_data(),
					active_variations_data = [],
					variation_id,
					is_compatible,
					variation;

				for ( var i = 0; i < variation_data.length; i++ ) {

					variation_id  = variation_data[ i ].variation_id.toString();
					is_compatible = _.contains( options_state.active, variation_id );

					// Copy all variation objects but set the variation_is_active property to false in order to disable the attributes of incompatible variations.
					// Only if WC v2.3 and disabled variations are set to be visible.
					if ( false === self.hide_disabled_variations() ) {

						var variation_has_empty_attributes = false;

						variation = $.extend( true, {}, variation_data[ i ] );

						if ( ! is_compatible ) {

							variation.variation_is_active = false;

							// Do not include incompatible variations with empty attributes - they can break stuff when prioritized.
							for ( var attr_name in variation.attributes ) {
								if ( variation.attributes[ attr_name ] === '' ) {
									variation_has_empty_attributes = true;
									break;
								}
							}
						}

						if ( ! variation_has_empty_attributes ) {
							active_variations_data.push( variation );
						}

					// Copy only compatible variations.
					// Only if disabled variations are set to be hidden.
					} else {
						if ( is_compatible ) {
							active_variations_data.push( variation_data[ i ] );
						} else {
							if ( parseInt( selected_variation, 10 ) === parseInt( variation_id, 10 ) ) {
								variation                     = $.extend( true, {}, variation_data[ i ] );
								variation.variation_is_active = false;
								active_variations_data.push( variation );
							}
						}
					}
				}

				// Save compatible variations.
				this.selected_product_data.active_variations_data = active_variations_data;
			}
		},

		/**
		 * True if the model has more state updates lined up.
		 */
		has_pending_updates: function() {

			if ( 'variable' === this.get_type() && false === this.get_variation_data() ) {
				return true;
			}

			return false;
		}

	} );

	var obj = new Model( opts );
	return obj;
};
