/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * Controls permission for access to a step.
 */
this.Step_Access_Model = function( step, opts ) {

	var self  = step;
	var Model = Backbone.Model.extend( {

		is_lockable: false,

		initialize: function() {

			var model  = this,
				params = {
				is_locked: false,
			};

			this.set( params );

			/*
			 * Permit lock state changes only if:
			 *
			 * - Layout !== 'Stacked'.
			 * - Layout !== 'Componentized', or 'composite_settings.sequential_componentized_progress' === 'yes'.
			 * - Layout === 'Componentized', 'composite_settings.sequential_componentized_progress' === 'yes' and this is not the Review step.
			 */
			this.is_lockable = composite.settings.layout !== 'single' && ( composite.settings.layout_variation !== 'componentized' || composite.settings.sequential_componentized_progress === 'yes' && false === self.is_review() );

			if ( this.is_lockable ) {

				$.each( composite.get_steps(), function( index, step ) {

					if ( step.is_review() ) {
						return true;
					}

					if ( step.step_index < self.step_index ) {
						// Update lock state when the validation state of a previous step changes.
						model.listenTo( step.step_validation_model, 'change:passes_validation', model.update_lock_state );
						// Update lock state when the lock state of a previous step changes.
						model.listenTo( step.step_access_model, 'change:is_locked', model.update_lock_state );
					}

				} );
			}

			/**
			 * Lock state also changes according to own step visibility.
			 */
			this.listenTo( self.step_visibility_model, 'change:is_visible', this.update_lock_state );
		},

		update_lock_state: function() {

			var lock = false;

			if ( false === self.is_visible() ) {
				lock = true;
			} else if ( this.is_lockable ) {

				$.each( composite.get_steps(), function( index, step ) {

					if ( step.step_index === self.step_index ) {
						return false;
					}

					if ( false === step.is_visible() ) {
						return true;
					}

					if ( step.step_access_model.get( 'is_locked' ) ) {
						lock = true;
						return false;
					} else if ( false === step.step_validation_model.get( 'passes_validation' ) ) {
						lock = true;
						return false;
					}

				} );
			}

			composite.console_log( 'debug:models', '\nUpdating \'Step_Access_Model\': "' + self.get_title() + '", Attribute: "is_locked": ' + lock.toString() );

			if ( this.get( 'is_locked' ) !== lock ) {
				composite.console_log( 'debug:models', 'Lock state changed.\n' );
			} else {
				composite.console_log( 'debug:models', 'Lock state unchanged.\n' );
			}

			composite.debug_indent_incr();
			this.set( { is_locked: lock } );
			composite.debug_indent_decr();
		}

	} );

	var obj = new Model( opts );
	return obj;
};
