/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * Validates the configuration state of a step.
 */
this.Step_Validation_Model = function( step, opts ) {

	var self  = step;
	var Model = Backbone.Model.extend( {

		initialize: function() {

			var params = {
				passes_validation:  true,
				is_in_stock:        true,
				component_messages: [],
				composite_messages: [],
			};

			this.set( params );

			/**
			 * Re-validate step when quantity is changed.
			 */
			composite.actions.add_action( 'component_quantity_changed', this.quantity_changed_handler, 10, this );

			/**
			 * Re-validate step when a new selection is made.
			 */
			composite.actions.add_action( 'component_selection_changed', this.selection_changed_handler, 20, this );

			/**
			 * Re-validate step when the contents of an existing selection change.
			 */
			composite.actions.add_action( 'component_selection_content_changed', this.selection_content_changed_handler, 20, this );
		},

		addons_changed_handler: function( step ) {

			if ( composite.is_initialized ) {

				if ( step.step_id !== self.step_id ) {
					return;
				}

				self.validate();
			}
		},

		quantity_changed_handler: function( step ) {

			if ( composite.is_initialized ) {

				if ( step.step_id !== self.step_id ) {
					return;
				}

				self.validate();
			}
		},

		selection_changed_handler: function() {

			if ( composite.is_initialized ) {

				// Do not change the validation state when a Variable Product is loading.
				if ( self.is_component() ) {
					if ( self.component_selection_model.has_pending_updates() ) {
						return;
					}
				}

				self.validate();
			}
		},

		selection_content_changed_handler: function( step ) {

			if ( composite.is_initialized ) {

				if ( step.step_id !== self.step_id ) {
					return;
				}

				self.validate();
			}
		},

		update: function( is_valid, is_in_stock ) {

			var params = {
				passes_validation:  is_valid,
				is_in_stock:        is_in_stock,
				component_messages: self.get_validation_messages( 'component' ),
				composite_messages: self.get_validation_messages( 'composite' )
			};

			composite.console_log( 'debug:models', '\nUpdating \'Step_Validation_Model\': "' + self.get_title() + '", Attribute: "passes_validation": ' + params.passes_validation.toString() + ', Attribute: "is_in_stock": ' + params.is_in_stock.toString() );

			if ( this.get( 'passes_validation' ) !== params.passes_validation ) {
				composite.console_log( 'debug:models', 'Validation state changed.\n' );
			} else {
				composite.console_log( 'debug:models', 'Validation state unchanged.\n' );
			}

			if ( ! _.isEqual( this.get( 'component_messages' ), params.component_messages ) ) {
				composite.console_log( 'debug:models', 'Validation message changed.\n' );
			} else {
				composite.console_log( 'debug:models', 'Validation message unchanged.\n' );
			}

			if ( this.get( 'is_in_stock' ) !== params.is_in_stock ) {
				composite.console_log( 'debug:models', 'Stock state changed.\n' );
			} else {
				composite.console_log( 'debug:models', 'Stock state unchanged.\n' );
			}

			composite.debug_indent_incr();
			this.set( params );
			composite.debug_indent_decr();
		}

	} );

	var obj = new Model( opts );
	return obj;
};
