/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * Updates the model data from UI interactions and listens to the component options model for updated content.
 */
this.Component_Options_View = function( component, opts ) {

	var self = component;
	var View = Backbone.View.extend( {

		templates: {
			dropdown:   false,
			thumbnails: false,
			radios:     false
		},

		update_action:    '',
		load_height:      0,
		$blocked_element: false,

		append_results_retry_count: 0,

		changes: {
			dropdown:   { changed: false, to: '' },
			thumbnails: { changed: false, to: '' },
			radios:     { changed: false, to: '' },
			variations: { changed: false, to: [] }
		},

		initialize: function() {

			this.templates.dropdown   = wp.template( 'wc_cp_options_dropdown' );
			this.templates.thumbnails = wp.template( 'wc_cp_options_thumbnails' );
			this.templates.radios     = wp.template( 'wc_cp_options_radio_buttons' );

			/**
		 	 * Reload component options upon activating a filter.
			 */
			self.$el.on( 'click', '.component_filter_option .toggle_filter_option', { view: this }, this.activate_filter );

			/**
			 * Reload component options upon resetting a filter.
			 */
			self.$el.on( 'click', '.component_filters .reset_component_filter', { view: this }, this.reset_filter );

			/**
			 * Reload component options upon resetting all filters.
			 */
			self.$el.on( 'click', '.component_filters .reset_component_filters', { view: this }, this.reset_filters );

			/**
			 * Reload component options upon reordering.
			 */
			self.$el.on( 'change', '.component_ordering select', { view: this }, this.order_by );

			/**
			 * Toggle filters.
			 */
			self.$el.on( 'click', '.component_filter_title .component_filter_name', { view: this }, this.toggle_filter );

			/**
			 * Navigate to step on clicking the blocked area in progressive mode.
			 */
			if ( composite.settings.layout === 'progressive' ) {
				self.$el.on( 'click', '.block_component_selections_inner', { view: this }, this.clicked_blocked_area );
			}

			/**
			 * Change selection when clicking a thumbnail or thumbnail tap area.
			 */
			if ( self.has_options_style( 'thumbnails' ) ) {
				self.$el.on( 'click', '.component_option_thumbnail_select', { view: this }, this.selected_thumbnail );
			}

			/**
			 * Change selection when clicking a radio button.
			 */
			if ( self.has_options_style( 'radios' ) ) {
				self.$el.on( 'change', '.component_option_radio_buttons input', { view: this }, this.clicked_radio );
			}

			/**
			 * Update view after appending/reloading component options.
			 */
			composite.actions.add_action( 'component_options_loaded_' + self.step_id, this.updated_options, 10, this );

			/**
			 * Render component options in view.
			 */
			composite.actions.add_action( 'component_options_state_changed_' + self.step_id, this.render, 10, this );

			/**
			 * Update component options in view when selections change.
			 */
			if ( 'relative' === self.get_price_display_format() ) {
				composite.actions.add_action( 'component_selection_changed', this.component_selection_changed_handler, 100, this );
				composite.actions.add_action( 'component_selection_content_changed', this.component_selection_changed_handler, 100, this );
			}

			/**
			 * Reload options if the scenarios used to render them have changed.
			 */
			this.listenTo( this.model, 'change:options_in_scenarios', this.options_in_scenarios_changed );
		},

		/**
		 * Updates relative prices in view when component totals change.
		 */
		component_selection_changed_handler: function( step ) {

			if ( self.step_id === step.step_id && ! self.component_selection_model.has_pending_updates() ) {
				self.component_options_view.render();
			}
		},

		options_in_scenarios_changed: function() {

			if ( this.model.reload_options_on_scenarios_change() ) {

				// Block options container.
				composite.block( self.$component_options );
				this.$blocked_element = self.$component_options;

				// Add status message.
				composite.data_model.add_status_message( self.component_id, wc_composite_params.i18n_loading_options.replace( '%s', self.get_title() ) );

				this.update_options( { page: 1 }, 'reload', true );
			}
		},

		clicked_blocked_area: function() {

			composite.navigate_to_step( self );
			return false;
		},

		selected_thumbnail: function() {

			var $button    = $( this ),
				$thumbnail = $button.closest( '.component_option_thumbnail' );

			if ( self.$el.hasClass( 'disabled' ) || $thumbnail.hasClass( 'disabled' ) ) {
				return true;
			}

			if ( ! $thumbnail.hasClass( 'selected' ) ) {
				var value = $thumbnail.data( 'val' );
				self.$component_options_select.val( value ).change();
			}

			return false;
		},

		clicked_radio: function() {

			var $this      = $( this ),
				$container = $this.closest( '.component_option_radio_button' );

			if ( self.$el.hasClass( 'disabled' ) || $container.hasClass( 'disabled' ) ) {
				return true;
			}

			if ( ! $container.hasClass( 'selected' ) ) {
				var value = $this.val();
				self.$component_options_select.val( value ).change();
			}
		},

		toggle_filter: function() {

			var $this                     = $( this ),
				$component_filter         = $this.closest( '.component_filter' ),
				$component_filter_content = $component_filter.find( '.component_filter_content' );

			wc_cp_toggle_element( $component_filter, $component_filter_content, false, 200 );

			return false;
		},

		activate_filter: function( event ) {

			var $this = $( this );

			// Do nothing if the component is disabled.
			if ( self.$el.hasClass( 'disabled' ) ) {
				return false;
			}

			var view                     = event.data.view,
				$component_filter_option = $this.closest( '.component_filter_option' );

			if ( ! $component_filter_option.hasClass( 'selected' ) ) {

				var $component_filter_container = $this.closest( '.component_filter' ),
					is_multiselect              = 'yes' === $component_filter_container.data( 'multiselect' );

				if ( ! is_multiselect ) {

					var $component_filter_options_selected = $component_filter_container.find( '.component_filter_option.selected' );

					$component_filter_options_selected.removeClass( 'selected' );
					$component_filter_options_selected.find( '.toggle_filter_option' ).attr( 'aria-checked', 'false' );
				}

				$component_filter_option.addClass( 'selected' );
				$component_filter_option.find( '.toggle_filter_option' ).attr( 'aria-checked', 'true' );


			} else {
				$component_filter_option.removeClass( 'selected' );
				$component_filter_option.find( '.toggle_filter_option' ).attr( 'aria-checked', 'false' );
			}

			// Add/remove 'active' classes.
			view.update_filters_ui();

			// Block container.
			composite.block( self.$component_filters );
			view.$blocked_element = self.$component_filters;

			view.update_options( { page: 1, filters: self.find_active_filters() }, 'reload' );

			return false;
		},

		reset_filter: function( event ) {

			// Get active filters.
			var $this                     = $( this ),
				view                      = event.data.view,
				$component_filter_options = $this.closest( '.component_filter' ).find( '.component_filter_option.selected' );

			if ( $component_filter_options.length == 0 ) {
				return false;
			}

			$component_filter_options.removeClass( 'selected' );
			$component_filter_options.find( '.toggle_filter_option' ).attr( 'aria-checked', 'false' );

			// Add/remove 'active' classes.
			view.update_filters_ui();

			// Block container.
			composite.block( self.$component_filters );
			view.$blocked_element = self.$component_filters;

			view.update_options( { page: 1, filters: self.find_active_filters() }, 'reload' );

			return false;
		},

		reset_filters: function( event ) {

			$( this ).blur();

			// Get active filters.
			var view                      = event.data.view,
				$component_filter_options = self.$component_filters.find( '.component_filter_option.selected' );

			if ( $component_filter_options.length == 0 ) {
				return false;
			}

			$component_filter_options.removeClass( 'selected' );
			$component_filter_options.find( '.toggle_filter_option' ).attr( 'aria-checked', 'false' );

			// Add/remove 'active' classes.
			view.update_filters_ui();

			// Block container.
			composite.block( self.$component_filters );
			view.$blocked_element = self.$component_filters;

			view.update_options( { page: 1, filters: self.find_active_filters() }, 'reload' );

			return false;
		},

		/**
		 * Add active/filtered classes to the component filters markup, can be used for styling purposes.
		 */
		update_filters_ui: function() {

			var $filters  = self.$component_filters.find( '.component_filter' ),
				all_empty = true;

			if ( $filters.length == 0 ) {
				return false;
			}

			$filters.each( function() {

				var $filter = $( this );

				if ( $filter.find( '.component_filter_option.selected' ).length == 0 ) {
					$filter.removeClass( 'active' );
				} else {
					$filter.addClass( 'active' );
					all_empty = false;
				}

			} );

			if ( all_empty ) {
				self.$component_filters.removeClass( 'filtered' );
			} else {
				self.$component_filters.addClass( 'filtered' );
			}
		},

		order_by: function( event ) {

			var $this   = $( this ),
				view    = event.data.view,
				orderby = $this.val();

			// Block container.
			composite.block( self.$component_options );
			view.$blocked_element = self.$component_options;

			view.update_options( { page: 1, orderby: orderby }, 'reload' );

			return false;
		},

		get_columns: function() {

			var columns = null;

			if ( self.has_options_style( 'thumbnails' ) ) {

				columns = parseInt( self.$component_thumbnail_options.data( 'component_option_columns' ), 10 );

				if ( isNaN( columns ) ) {
					columns = typeof self.$component_thumbnail_options.data( 'columns' ) !== 'undefined' ? parseInt( self.$component_thumbnail_options.data( 'columns' ), 10 ) : 1;
				}

			} else if ( self.has_options_style( 'radios' ) ) {
				columns = 1;
			}

			return columns;
		},

		/**
		 * Renders options in the DOM based on 'active_options' model attribute changes.
		 */
		render: function( dropdown_only ) {

			if ( ! composite.is_initialized ) {
				return false;
			}

			dropdown_only = typeof( dropdown_only ) === 'undefined' ? false : dropdown_only;

			composite.console_log( 'debug:views', '\nRendering "' + self.get_title() + '" options in view...' );

			var model            = self.component_options_model,
				price_format     = self.get_price_display_format(),
				active_options   = model.get( 'options_state' ).active,
				selected_product = self.get_selected_product( false ),
				options_data     = $.extend( true, [], model.available_options_data ),
				change_what      = [];

			this.changes.dropdown.changed   = false;
			this.changes.thumbnails.changed = false;
			this.changes.radios.changed     = false;
			this.changes.variations.changed = false;

			/*
			 * Hide or grey-out inactive products.
			 */

			$.each( options_data, function( index, option_data ) {

				var product_id    = option_data.option_id,
					is_compatible = _.contains( active_options, product_id );

				if ( ! is_compatible ) {
					options_data[ index ].is_disabled = true;
				} else {
					options_data[ index ].is_disabled = false;
				}

				options_data[ index ].is_hidden   = options_data[ index ].is_disabled && self.hide_disabled_products();
				options_data[ index ].is_selected = options_data[ index ].option_id === selected_product;

				options_data[ index ].option_button_text   = option_data.is_configurable ? wc_composite_params.i18n_configure_option_button : wc_composite_params.i18n_select_option_button;
				options_data[ index ].option_button_label  = wc_composite_params.i18n_option_action_label.replace( '%a', options_data[ index ].option_button_text ).replace( '%o', option_data.option_title );
				options_data[ index ].option_display_title = option_data.option_title;
				options_data[ index ].option_price_html    = self.get_formatted_option_price_html( options_data[ index ] );
			} );


			// Dropdown template data.
			var dropdown_options_data = $.extend( true, [], options_data );

			$.each( dropdown_options_data, function( index, option_data ) {

				dropdown_options_data[ index ].is_selected = option_data.is_selected && self.is_selected_product_valid();

				// Only append price if visible.
				if ( option_data.option_price_html && self.has_options_style( 'dropdowns' ) ) {

					var i18n_option_display_title_string = 'relative' === price_format ? wc_composite_params.i18n_dropdown_title_relative_price : wc_composite_params.i18n_dropdown_title_price;

					dropdown_options_data[ index ].option_display_title = i18n_option_display_title_string.replace( '%t', option_data.option_display_title ).replace( '%p', option_data.option_price_html );
				}
			} );

			var show_empty_option     = false,
				show_switching_option = false,
				empty_option_disabled = false,
				empty_option_title;

			// Always add an empty option when there are no valid options to select - necessary to allow resetting an existing invalid selection.
			if ( active_options.length === 0 ) {

				show_empty_option  = true;
				empty_option_title = wc_composite_params.i18n_no_options;

			} else {

				empty_option_title = self.is_optional() ? wc_composite_params.i18n_no_option.replace( '%s', self.get_title() ) : wc_composite_params.i18n_select_option.replace( '%s', self.get_title() );

				if ( self.maybe_is_optional() ) {

					show_empty_option = true;

					if ( false === self.is_selected_product_valid() ) {
						show_switching_option = true;
					}

					if ( false === self.is_optional() ) {

						if ( '' === selected_product ) {
							show_switching_option = true;
						}

						empty_option_disabled = true;
						empty_option_title    = wc_composite_params.i18n_no_option.replace( '%s', self.get_title() );
					}

				} else if ( false === self.is_static() && self.show_placeholder_option() ) {
					show_empty_option = true;
				} else if ( '' === selected_product && false === self.show_placeholder_option() ) {
					show_empty_option = true;
				} else if ( false === self.is_selected_product_valid() && false === self.show_placeholder_option() ) {
					show_switching_option = true;
				}
			}

			if ( show_empty_option ) {

				var empty_option_data = {
					option_id:            '',
					option_display_title: empty_option_title,
					is_disabled:          empty_option_disabled,
					is_hidden:            empty_option_disabled && self.hide_disabled_products(),
					is_selected:          selected_product === '' && false === show_switching_option
				};

				if ( 'relative' === price_format && self.has_options_style( 'dropdowns' ) && self.maybe_is_optional() ) {

					empty_option_data.option_price_html = '';
					empty_option_data.option_price_data = {
						price:             0.0,
						regular_price:     0.0,
						max_price:         0.0,
						max_regular_price: 0.0,
						min_qty:           1,
						discount:          ''
					};

					var empty_option_price_html = self.get_formatted_option_price_html( empty_option_data );

					if ( empty_option_price_html ) {
						empty_option_data.option_display_title = wc_composite_params.i18n_dropdown_title_relative_price.replace( '%t', empty_option_data.option_display_title ).replace( '%p', empty_option_price_html );
					}
				}

				dropdown_options_data.unshift( empty_option_data );
			}

			if ( show_switching_option ) {
				dropdown_options_data.unshift( {
					option_id:            '',
					option_display_title: wc_composite_params.i18n_select_option.replace( '%s', self.get_title() ),
					is_disabled:          false,
					is_hidden:            false,
					is_selected:          false
				} );
			}

			// Render Dropdown template.
			this.changes.dropdown.changed = true;
			this.changes.dropdown.to      = this.templates.dropdown( dropdown_options_data );

			if ( false === dropdown_only ) {

				// Thumbnails template.
				if ( self.has_options_style( 'thumbnails' ) ) {

					var thumbnail_options_data = _.where( options_data, { is_in_view: true } ),
						thumbnail_columns      = this.get_columns(),
						thumbnail_loop         = 0;

					if ( thumbnail_options_data.length > 0 ) {

						$.each( thumbnail_options_data, function( index, option_data ) {

							thumbnail_options_data[ index ].outer_classes  = option_data.is_hidden ? 'hidden' : '';
							thumbnail_options_data[ index ].inner_classes  = option_data.is_disabled ? 'disabled' : '';
							thumbnail_options_data[ index ].inner_classes += option_data.option_id === selected_product ? ' selected' : '';
							thumbnail_options_data[ index ].inner_classes += option_data.is_appended ? ' appended' : '';

							if ( false === option_data.is_hidden ) {

								thumbnail_loop++;

								if ( ( ( thumbnail_loop - 1 ) % thumbnail_columns ) == 0 || thumbnail_columns == 1 ) {
									thumbnail_options_data[ index ].outer_classes += ' first';
								}

								if ( thumbnail_loop % thumbnail_columns == 0 ) {
									thumbnail_options_data[ index ].outer_classes += ' last';
								}
							}
						} );
					}

					// Render Thumbnails template.
					var new_template_html = this.templates.thumbnails( thumbnail_options_data );

					// Ignore 'selected' class changes in comparison.
					if ( new_template_html.replace( / selected/g, '' ) !== this.changes.thumbnails.to.replace( / selected/g, '' ) ) {
						this.changes.thumbnails.changed = true;
						this.changes.thumbnails.to      = new_template_html;
					} else {
						composite.console_log( 'debug:views', '...skipped!' );
					}

				// Radio buttons template.
				} else if ( self.has_options_style( 'radios' ) ) {

					var radio_options_data  = _.where( options_data, { is_in_view: true } ),
						show_empty_radio    = false,
						disable_empty_radio = self.maybe_is_optional() && false === self.is_optional(),
						hide_empty_radio    = disable_empty_radio && self.hide_disabled_products();

					if ( self.maybe_is_optional() ) {
						show_empty_radio = true;
					} else if ( false === self.is_static() && self.show_placeholder_option() ) {
						show_empty_radio = true;
						hide_empty_radio = true;
					}

					if ( show_empty_radio ) {

						var empty_radio_data = {
							option_id:             '',
							option_display_title:  wc_composite_params.i18n_no_option.replace( '%s', self.get_title() ),
							is_disabled:           disable_empty_radio,
							is_hidden:             hide_empty_radio,
							is_selected:           selected_product === ''
						};

						if ( 'relative' === price_format && self.maybe_is_optional() ) {

							empty_radio_data.option_price_html = '';
							empty_radio_data.option_price_data = {
								price:             0.0,
								regular_price:     0.0,
								max_price:         0.0,
								max_regular_price: 0.0,
								min_qty:           1,
								discount:          ''
							};

							var empty_radio_price_html = self.get_formatted_option_price_html( empty_radio_data );

							if ( empty_radio_price_html ) {
								empty_radio_data.option_price_html = empty_radio_price_html;
							}
						}

						radio_options_data.unshift( empty_radio_data );
					}

					if ( radio_options_data.length > 0 ) {

						$.each( radio_options_data, function( index, option_data ) {

							radio_options_data[ index ].outer_classes  = option_data.is_hidden ? 'hidden' : '';
							radio_options_data[ index ].inner_classes  = option_data.is_disabled ? 'disabled' : '';
							radio_options_data[ index ].inner_classes += option_data.option_id === selected_product ? ' selected' : '';

							radio_options_data[ index ].option_suffix   = option_data.option_id === '' ? '0' : option_data.option_id;
							radio_options_data[ index ].option_group_id = self.component_id;
						} );
					}

					// Render Radio buttons template.
					this.changes.radios.changed = true;
					this.changes.radios.to      = this.templates.radios( radio_options_data );
				}

				/*
				 * Hide or grey-out inactive variations.
				 */

				if ( 'variable' === self.get_selected_product_type() ) {

					var compatible_variation_data = self.component_selection_model.get_active_variations_data();

					this.changes.variations.changed = ! _.isEqual( this.changes.variations.to, compatible_variation_data ) && this.changes.variations.to.length > 0;
					this.changes.variations.to      = compatible_variation_data;
				}
			}

			change_what = _.keys( _.pick( this.changes, function( value ) { return value.changed; } ) );

			if ( change_what.length > 0 ) {

				// Run 'component_options_state_render' action - @see WC_CP_Composite_Dispatcher class.
				composite.actions.do_action( 'component_options_state_render', [ self, change_what ] );

				if ( this.changes.dropdown.changed ) {
					self.$component_options_select.html( this.changes.dropdown.to );
				}

				if ( this.changes.thumbnails.changed ) {

					self.$component_thumbnail_options.html( this.changes.thumbnails.to );

					if ( 'reload' === this.update_action ) {
						self.$component_selections.removeClass( 'refresh_component_options' );
					}
				}

				if ( this.changes.radios.changed ) {
					self.$component_radio_button_options.html( this.changes.radios.to );
				}

				if ( this.changes.variations.changed ) {

					composite.console_log( 'debug:views', '\nVariations data in "' + self.get_title() + '" has changed. Reinitializing variations view...' );

					// Update WC variations model state.
					self.$component_summary_content.data( 'product_variations', self.component_selection_model.get_active_variations_data() );
					// Update the variations script.
					composite.debug_indent_incr();
					self.$component_summary_content.triggerHandler( 'reload_product_variations' );
					composite.debug_indent_decr();
				}

				// Run 'component_options_state_rendered' action - @see WC_CP_Composite_Dispatcher class.
				composite.actions.do_action( 'component_options_state_rendered', [ self, change_what ] );
			}
		},

		/**
		 * Update options after collecting user input.
		 */
		update_options: function( params, update_action, is_background_request ) {

			is_background_request = typeof( is_background_request ) === 'undefined' ? false : is_background_request;

			this.update_action = update_action;

			if ( 'reload' === update_action ) {
				self.$component_selections.addClass( 'refresh_component_options' );
			}

			this.load_height = self.$component_options.get( 0 ).getBoundingClientRect().height;

			if ( typeof this.load_height === 'undefined' ) {
				this.load_height = self.$component_options.outerHeight();
			}

			// Lock height.
			self.$component_options.css( 'height', this.load_height );

			setTimeout( function() {
				self.component_options_model.request_options( params, update_action );
			}, 200 );

			// Run 'component_options_update_requested' action - @see WC_CP_Composite_Dispatcher class.
			composite.actions.do_action( 'component_options_update_requested', [ self, params, update_action, is_background_request ] );
		},

		/**
		 * Update view after appending/reloading component options.
		 */
		updated_options: function() {

			if ( false === this.$blocked_element ) {
				return false;
			}

			if ( 'append' === this.update_action && self.hide_disabled_products() ) {

				if ( self.$component_thumbnail_options.find( '.appended:not(.disabled)' ).length < self.get_results_per_page() ) {

					var retry = this.model.get( 'page' ) < this.model.get( 'pages' );

					if ( retry && this.append_results_retry_count > 10 ) {
						if ( false === window.confirm( wc_composite_params.i18n_reload_threshold_exceeded.replace( '%s', self.get_title() ) ) ) {
							retry = false;
						}
					}

					if ( retry ) {
						this.append_results_retry_count++;
						this.model.request_options( { page: this.model.get( 'page' ) + 1 }, 'append' );
						return false;
					} else {
						this.append_results_retry_count = 0;
					}
				}
			}

			// Preload images before proceeding.
			var $thumbnails_container = self.$component_thumbnail_options.find( '.component_option_thumbnails_container' ),
				$thumbnail_images     = $thumbnails_container.find( '.component_option_thumbnail_container:not(.hidden) img' ),
				view                  = this,
				task                  = new wc_cp_classes.WC_CP_Async_Task( function() {

				var wait       = false,
					async_task = this;

				if ( $thumbnail_images.length > 0 && $thumbnails_container.is( ':visible' ) ) {
					$thumbnail_images.each( function() {
						if ( $( this ).height() === 0 && async_task.get_async_time() < 20000 ) {
							wait = true;
							return false;
						}
					} );
				}

				if ( ! wait ) {
					this.done();
				}

			}, 50 );

			task.complete( function() {
				view.animate_options();
			} );
		},

		/**
		 * Animate view when reloading/appending options.
		 */
		animate_options: function() {

			if ( 'append' === this.update_action ) {
				appended = self.$component_thumbnail_options.find( '.appended' );
				appended.removeClass( 'appended' );
			}

			var view           = this,
				new_height     = self.$component_options_inner.outerHeight( true ),
				animate_height = false;

			if ( Math.abs( new_height - view.load_height ) > 1 ) {
				animate_height = true;
			} else {
				self.$component_options.css( 'height', 'auto' );
			}

			var appended = {};

			// Animate component options container.
			if ( animate_height ) {

				composite.console_log( 'debug:animations', 'Starting component options height animation...' );

				self.$component_options.animate( { 'height' : new_height }, { duration: 250, queue: false, always: function() {
					self.$component_options.css( { 'height' : 'auto' } );

					composite.console_log( 'debug:animations', 'Ended component options height animation.' );

					setTimeout( function() {
						view.unblock();
					}, 100 );

				} } );

			} else {
				setTimeout( function() {
					view.unblock();
				}, 250 );
			}

			// Run 'component_options_updated' action - @see WC_CP_Actions_Dispatcher class.
			composite.actions.do_action( 'component_options_updated', [ self ] );
		},

		/**
		 * Unblock blocked view element.
		 */
		unblock: function() {

			self.$component_selections.removeClass( 'refresh_component_options' );

			composite.unblock( this.$blocked_element );

			this.$blocked_element = false;

			// Remove status message.
			composite.data_model.remove_status_message( self.component_id );

			this.update_action = '';

			composite.actions.do_action( 'component_options_refreshed', [ self ] );
		},

		/**
		 * True if the view is updating.
		 */
		is_updating: function() {

			return false !== this.$blocked_element;
		}

	} );

	var obj = new View( opts );
	return obj;
};
