/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View associated with the component options pagination template.
 */
this.Component_Pagination_View = function( component, opts ) {

	var self = component;
	var View = Backbone.View.extend( {

		template: false,

		initialize: function() {

			this.template = wp.template( 'wc_cp_options_pagination' );

			/**
		 	 * Update the view when its model state changes.
			 */
			this.listenTo( this.model, 'change:page change:pages', this.render );

			/**
			 * Reload component options upon requesting a new page.
			 */
			self.$el.on( 'click', '.component_pagination a.component_pagination_element', { view: this }, this.load_page );

			/**
			 * Append component options upon clicking the 'Load More' button.
			 */
			self.$el.on( 'click', '.component_pagination .component_options_load_more', { view: this }, this.load_more );

		},

		load_page: function() {

			var page = parseInt( $( this ).data( 'page_num' ), 10 );

			if ( page > 0 ) {

				// Block container.
				composite.block( self.$component_options );
				self.component_options_view.$blocked_element = self.$component_options;

				self.component_options_view.update_options( { page: page }, 'reload' );
			}

			return false;
		},

		load_more: function() {

			var page  = parseInt( self.component_options_model.get( 'page' ), 10 ),
				pages = parseInt( self.component_options_model.get( 'pages' ), 10 );

			if ( page > 0 && page < pages ) {

				// Block container.
				composite.block( self.$component_options );
				self.component_options_view.$blocked_element = self.$component_options;

				self.component_options_view.update_options( { page: page + 1 }, 'append' );
			}

			return false;
		},

		/**
		 * Renders the view.
		 */
		render: function() {

			if ( ! composite.is_initialized ) {
				return false;
			}

			var	model = this.model,
				data  = {
					page:                model.get( 'page' ),
					pages:               model.get( 'pages' ),
					range_mid:           self.get_pagination_range(),
					range_end:           self.get_pagination_range( 'end' ),
					pages_in_range:      ( ( self.get_pagination_range() + self.get_pagination_range( 'end' ) ) * 2 ) + 1,
					i18n_page_of_pages:  wc_composite_params.i18n_page_of_pages.replace( '%p', model.get( 'page' ) ).replace( '%t', model.get( 'pages' ) )
				};

			composite.console_log( 'debug:views', '\nRendering "' + self.get_title() + '" options pagination...' );

			if ( self.append_results() ) {

				if ( data.page < data.pages ) {
					this.$el.slideDown( 200 );
				} else {
					this.$el.slideUp( 200 );
				}

			} else {

				this.$el.html( this.template( data ) );
			}
		}

	} );

	var obj = new View( opts );
	return obj;
};
