/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* global wc_cp_get_variation_data */
/* @endexclude */

/**
 * Updates the model data from UI interactions and listens to the component selection model for updated content.
 */
this.Component_Selection_View = function( component, opts ) {

	var self = component;
	var	View = Backbone.View.extend( {

		templates:                    {},

		$relocation_origin:           false,
		relocated:                    false,

		relocating:                   false,
		relocating_to_origin:         false,
		$relocation_target:           false,
		$relocation_reference:        false,
		load_height:                  0,

		flushing_component_options:   false,

		rendered_product:             '',

		initialize: function() {

			this.templates = {
				selection_title:      wp.template( 'wc_cp_component_selection_title' ),
				selection_title_html: ''
			};

			/**
			 * Update model on changing a component option.
			 */
			self.$el.on( 'change', '.component_options select.component_options_select', { view: this }, this.option_changed );

			/**
			 * Update model data when a new variation is selected.
			 */
			self.$el.on( 'woocommerce_variation_has_changed', { view: this }, function( event ) {

				var variation_id   = self.$component_summary_content.find( '.single_variation_wrap .variations_button input.variation_id' ).val(),
					$variations    = self.$component_summary_content.find( '.variations' ),
					variation_data = {
						meta_data: []
					};

				variation_id = variation_id || '';

				if ( variation_id && $variations.length > 0 ) {
					variation_data.meta_data = wc_cp_get_variation_data( $variations, false );
				}

				// Update model.
				event.data.view.model.update_selected_variation( variation_id, variation_data );

				if ( self.$component_variations_reset_wrapper ) {
					if ( event.data.view.model.get( 'selected_variation' ) ) {
						self.$component_variations_reset_wrapper.slideDown( 200 );
					} else {
						self.$component_variations_reset_wrapper.slideUp( 200 );
					}
				}

				if ( self.step_validation_model.get( 'is_in_stock' ) ) {
					// Ensure min/max constraints are always honored.
					self.$component_quantity.trigger( 'change' );
				}

				// Remove images class from composited_product_images div in order to avoid styling issues.
				if ( ! self.has_wc_core_gallery_class ) {
					self.$component_summary_content.find( '.composited_product_images' ).removeClass( 'images' );
				}
			} );

			/**
			 * Add 'images' class to composited_product_images div when initiating a variation selection change.
			 */
			self.$el.on( 'woocommerce_variation_select_change', function() {

				// Required by the variations script to flip images.
				if ( ! self.has_wc_core_gallery_class ) {
					self.$component_summary.find( '.composited_product_images' ).addClass( 'images' );
				}

				// Reset component prices.
				self.component_selection_model.set_price( 0.0 );
				self.component_selection_model.set_regular_price( 0.0 );
				self.component_selection_model.set_tax_ratios( false );

				// Reset availability.
				self.component_selection_model.set_stock_status( '' );

			} );

			/**
			 * Update composite totals and form inputs when a new variation is selected.
			 */
			self.$el.on( 'found_variation', function( event, variation ) {

				// Update component prices.
				self.component_selection_model.set_price( variation.price );
				self.component_selection_model.set_regular_price( variation.regular_price );
				self.component_selection_model.set_tax_ratios( variation.tax_ratios );

				// Update availability.
				if ( ! variation.is_in_stock ) {
					self.component_selection_model.set_stock_status( 'out-of-stock' );
				}

			} );

			/**
			 * Update model upon changing quantities.
			 */
			self.$el.on( 'input change', '.component_wrap input.qty', function( e ) {

				var $input = $( this ),
					min    = parseFloat( $input.attr( 'min' ) ),
					max    = parseFloat( $input.attr( 'max' ) );

				if ( wc_composite_params.force_min_max_qty_input === 'yes' ) {
					if ( 'change' === e.type && min >= 0 && ( parseFloat( $input.val() ) < min || isNaN( parseFloat( $input.val() ) ) ) ) {
						$input.val( min );
					}

					if ( 'change' === e.type && max > 0 && parseFloat( $input.val() ) > max ) {
						$input.val( max );
					}
				}

				self.component_selection_model.update_selected_quantity( parseInt( $input.val() ) );
			} );

			/**
			 * Initialize prettyPhoto/phptoSwipe script when component selection scripts are initialized.
			 */
			self.$el.on( 'wc-composite-component-loaded', { view: this }, function() {

				// Init PhotoSwipe if present.
				if ( 'yes' === wc_composite_params.photoswipe_enabled && typeof PhotoSwipe !== 'undefined' ) {

					var $product_image = self.$component_summary_content.find( '.composited_product_images' );

					$product_image.wc_product_gallery( { zoom_enabled: false, flexslider_enabled: false } );

					var $placeholder = $product_image.find( 'a.placeholder_image' );

					if ( $placeholder.length > 0 ) {
						$placeholder.on( 'click', function() {
							return false;
						} );
					}

				// Otherwise, fall back to prettyPhoto.
				} else if ( $.isFunction( $.fn.prettyPhoto ) ) {

					var $prettyphoto_images = self.$component_summary_content.find( 'a[data-rel^="prettyPhoto"]' ),
						$active_images      = $prettyphoto_images.not( '.placeholder_image' ),
						$inactive_images    = $prettyphoto_images.filter( '.placeholder_image' );

					if ( $active_images.length > 0 ) {
						$active_images.prettyPhoto( {
							hook: 'data-rel',
							social_tools: false,
							theme: 'pp_woocommerce',
							horizontal_padding: 20,
							opacity: 0.8,
							deeplinking: false
						} );
					}

					if ( $inactive_images.length > 0 ) {
						$inactive_images.on( 'click', function() {
							return false;
						} );
					}
				}
			} );

			/**
			 * On clicking the clear options button.
			 */
			self.$el.on( 'click', '.clear_component_options', function() {

				if ( $( this ).hasClass( 'reset_component_options' ) ) {
					return false;
				}

				var empty_option = self.$component_options_select.find( 'option[value=""]' );

				if ( empty_option.length > 0 && false === empty_option.first().prop( 'disabled' ) ) {
					self.$component_options_select.val( '' ).change();
				}

				return false;
			} );

			/**
			 * On clicking the reset options button.
			 */
			self.$el.on( 'click', '.reset_component_options', function() {

				var empty_option = self.$component_options_select.find( 'option[value=""]' );

				self.unblock_step_inputs();

				self.set_active();

				if ( empty_option.length > 0 && false === empty_option.first().prop( 'disabled' ) ) {
					self.$component_options_select.val( '' ).change();
				}

				self.block_next_steps();

				return false;
			} );

			/**
			 * Update model upon changing addons selections.
			 */
			self.$el.on( 'updated_addons', this.updated_addons_handler );

			/**
			 * Update composite totals when a new NYP price is entered.
			 */
			self.$el.on( 'woocommerce-nyp-updated-item', this.updated_nyp_handler );

			/*
			 * When entering a component with relocated selection details,
			 * reset the position of the relocated container if the 'relocated_content_reset_on_return' flag is set to 'yes'.
			 */
			if ( wc_composite_params.relocated_content_reset_on_return === 'yes' ) {
				composite.actions.add_action( 'active_step_transition', this.active_step_transition_handler, 100, this );
			}

			/**
			 * Update "Clear selection" button in view.
			 */
			composite.actions.add_action( 'component_options_state_changed_' + self.step_id, this.options_state_changed_handler, 100, this );

			/*
			 * When rendering a component with relocated selection details,
			 * back up and put back the relocated container after rendering the JS template contents.
			 */
			composite.actions.add_action( 'component_options_state_render', this.options_state_render_handler, 10, this );
			composite.actions.add_action( 'component_options_state_rendered', this.options_state_rendered_handler, 10, this );

			// Auto-select variable product attributes.
			composite.actions.add_action( 'component_options_state_rendered', this.autoselect_attributes, 30, this );

			/**
			 * Update the selection title when the product selection view content is changed.
			 */
			composite.actions.add_action( 'component_selection_details_updated_' + self.step_id, this.refresh_selection_title, 10, this );

			/**
			 * Render selection details into view.
			 */
			composite.actions.add_action( 'component_selection_changed', this.component_selection_changed_handler, 100, this );

			if ( self.maybe_autotransition() ) {
				composite.actions.add_action( 'active_step_transition_end', this.active_step_transition_end_handler, 10, this );
			}

			this.listenTo( this.model, 'selected_product_data_load_error', this.selection_data_load_error );

			/**
			 * Reset relocated content before flushing outdated component options.
			 */
			this.listenTo( self.component_options_model, 'component_options_data_loaded', this.component_options_flush_handler );

			/**
			 * Update the selection title when the quantity is changed.
			 */
			composite.actions.add_action( 'component_quantity_changed', this.quantity_changed_handler, 100, this );
		},

		/**
		 * Allows filtering animation durations.
		 */
		get_animation_duration: function( open_or_close ) {

			var duration = ( self.is_current() || 'single' === composite.settings.layout ) ? 220 : 0;

			open_or_close = open_or_close !== 'open' && open_or_close !== 'close' ? 'open' : open_or_close;

			// Pass through 'component_animation_duration' filter - @see WC_CP_Filters_Manager class.
			return composite.filters.apply_filters( 'component_selection_change_animation_duration', [ duration, open_or_close, self ] );
		},

		/**
		 * Resets the position of the relocated container when the active step changes.
		 */
		active_step_transition_handler: function( step ) {

			if ( self.step_id === step.step_id ) {
				if ( this.is_relocated() ) {
					this.reset_relocated_content();
				}
			}
		},

		/**
		 * Redraw the selection title.
		 */
		options_state_changed_handler: function() {

			if ( ! composite.is_initialized ) {
				return;
			}

			if ( this.rendered_product !== this.model.get( 'selected_product' ) ) {
				return;
			}

			this.update_selection_title();
		},

		/**
		 * Backup the relocated container before rendering the thumbnails view.
		 */
		options_state_render_handler: function( step, changed ) {

			if ( self.step_id === step.step_id ) {

				if ( _.contains( changed, 'thumbnails' ) && this.is_relocated() ) {

					// Save component content.
					self.$el.append( this.$relocation_target.hide() );
				}
			}
		},

		/**
		 * Put back the relocated container after rendering thumbnails view.
		 */
		options_state_rendered_handler: function( step, changed ) {

			if ( self.step_id === step.step_id ) {

				if ( _.contains( changed, 'thumbnails' ) && this.is_relocated() ) {

					var relocation_params = this.get_new_relocation_data();

					if ( relocation_params.relocate ) {

						this.$relocation_reference = relocation_params.reference;

						this.$relocation_reference.after( this.$relocation_target );

						this.$relocation_target.show();
					}
				}
			}
		},

		/**
		 * Auto-select variable product attributes.
		 */
		autoselect_attributes: function( step, changed ) {

			if ( self.step_id === step.step_id && step.autoselect_attributes() ) {

				if ( _.contains( changed, 'variations' ) ) {

					var active_variations = _.where( self.component_options_view.changes.variations.to, { variation_is_active: true } );

					if ( active_variations.length === 1 ) {

						var attributes                     = active_variations[ 0 ].attributes,
							variation_has_empty_attributes = false;

						for ( var attr_name in attributes ) {
							if ( attributes[ attr_name ] === '' ) {
								variation_has_empty_attributes = true;
								break;
							}
						}

						if ( ! variation_has_empty_attributes ) {

							var $attribute_fields       = self.$component_summary_content.find( '.variations select' ),
								attribute_field_val_set = false;

							$attribute_fields.each( function( index, el ) {

								var $current_attribute_field = $( el ),
									$attr_option_with_val    = $current_attribute_field.val() ? false : $current_attribute_field.find( 'option.attached.enabled[value!=""]:first' ),
									attr_val_to_set          = $attr_option_with_val ? $attr_option_with_val.val() : false;

								if ( attr_val_to_set ) {
									attribute_field_val_set = true;
									$current_attribute_field.val( attr_val_to_set );
								}
							} );

							if ( attribute_field_val_set ) {
								$attribute_fields.last().change();
							}
						}
					}
				}
			}
		},

		/**
		 * Updates the selection title when the quantity is changed.
		 */
		quantity_changed_handler: function( step ) {

			if ( step.step_id === self.step_id ) {
				this.update_selection_title( this.model );
			}
		},

		/**
		 * Renders addons totals and updates the model upon changing addons selections.
		 */
		updated_addons_handler: function( event ) {

			var addons_raw_price = 0,
				addons_data      = [],
				tax_ratios       = composite.data_model.price_data.price_tax_ratios[ self.component_id ];

			if ( ! self.has_addons() ) {
				return;
			}

			if ( wc_composite_params.pao_version_gte_30 ) {

				addons_data = self.$component_addons_totals.data( 'price_data' );

				$.each( addons_data, function( i, addon ) {

					if ( addon.is_custom_price ) {

						var tax_ratio_incl = tax_ratios && typeof( tax_ratios.incl ) !== 'undefined' ? Number( tax_ratios.incl ) : false,
							tax_ratio_excl = tax_ratios && typeof( tax_ratios.excl ) !== 'undefined' ? Number( tax_ratios.excl ) : false;

						if ( 'incl' === wc_composite_params.tax_display_shop && 'no' === wc_composite_params.prices_include_tax ) {
							addons_raw_price += addon.cost_raw / ( tax_ratio_incl ? tax_ratio_incl : 1 );
						} else if ( 'excl' === wc_composite_params.tax_display_shop && 'yes' === wc_composite_params.prices_include_tax ) {
							addons_raw_price += addon.cost_raw / ( tax_ratio_excl ? tax_ratio_excl : 1 );
						} else {
							addons_raw_price += addon.cost_raw;
						}

						// Custom Price fields always behave as Flat-Fee.
						addons_raw_price = addons_raw_price / self.get_selected_quantity();

					} else {

						if ( 'quantity_based' === addon.price_type ) {
							addons_raw_price += addon.cost_raw_pu;
						} else if ( 'flat_fee' === addon.price_type ) {
							addons_raw_price += addon.cost_raw / self.get_selected_quantity();
						} else if ( 'percentage_based' === addon.price_type ) {
							addons_raw_price += addon.cost_raw_pct * composite.data_model.price_data.prices[ self.component_id ];
						}
					}

				} );

			} else {

				var $addons = self.find( '.product-addon input, .product-addon textarea, .product-addon textarea, .product-addon select' );

				if ( $addons.length > 0 ) {
					$addons.each( function() {
						addons_data.push( $( this ).val() );
					} );
				}

				addons_raw_price = self.$component_addons_totals.data( 'addons-raw-price' ) || 0;

				if ( self.has_custom_price_addons() ) {

					self.$custom_price_addons.each( function() {

						var $addon         = $( this ),
							tax_ratio_incl = tax_ratios && typeof( tax_ratios.incl ) !== 'undefined' ? Number( tax_ratios.incl ) : false,
							tax_ratio_excl = tax_ratios && typeof( tax_ratios.excl ) !== 'undefined' ? Number( tax_ratios.excl ) : false;

						if ( 'incl' === wc_composite_params.tax_display_shop && 'no' === wc_composite_params.prices_include_tax ) {
							addons_raw_price += $addon.val() / ( tax_ratio_incl ? tax_ratio_incl : 1 );
						} else if ( 'excl' === wc_composite_params.tax_display_shop && 'yes' === wc_composite_params.prices_include_tax ) {
							addons_raw_price += $addon.val() / ( tax_ratio_excl ? tax_ratio_excl : 1 );
						} else {
							addons_raw_price += $addon.val();
						}

					} );
				}
			}

			addons_raw_price = addons_raw_price || 0.0;

			self.component_selection_model.update_selected_addons( addons_data, addons_raw_price );

			var price_data   = composite.data_model.price_data,
				addons_price = price_data.addons_prices[ self.component_id ];

			addons_price = addons_price || 0.0;

			if ( self.show_addons_totals ) {

				if ( self.passes_validation() ) {

					var qty           = self.get_selected_quantity(),
						addons_totals = composite.data_model.get_taxed_totals( addons_price, addons_price, tax_ratios, qty );

					if ( addons_totals.price > 0 ) {

						var price              = Number( price_data.prices[ self.component_id ] ),
							total              = price + Number( addons_price ),
							totals             = composite.data_model.get_taxed_totals( total, total, tax_ratios, qty ),
							price_html         = wc_cp_price_format( totals.price ),
							price_html_suffix  = composite.composite_price_view.get_formatted_price_suffix( totals ),
							addons_totals_html = '<span class="price">' + '<span class="subtotal">' + wc_composite_params.i18n_subtotal + '</span>' + price_html + price_html_suffix + '</span>';

						self.$component_addons_totals.html( addons_totals_html );

					} else {
						self.$component_addons_totals.empty();
					}

				} else {
					self.$component_addons_totals.empty();
				}
			}

			event.stopPropagation();
		},

		/**
		 * Updates the composite data model upon changing addons selections.
		 */
		updated_nyp_handler: function() {

			var $nyp      = self.$component_summary_content.find( '.nyp' ),
				nyp_price = $nyp.length > 0 ? Number( $nyp.data( 'price' ) ) : 0;

			self.component_selection_model.update_nyp( nyp_price );
		},

		/**
		 * Refreshes the selection title every time it changes.
		 */
		refresh_selection_title: function() {

			// Clear selection title template to resolve an issue with rendering after clearing and selecting the same product.
			this.templates.selection_title_html = '';
			// Update title.
			this.update_selection_title( this.model );
		},

		/**
		 * Renders the selected product title and the "Clear selection" button.
		 */
		update_selection_title: function( model ) {

			var view = this;

			model = typeof( model ) === 'undefined' ? view.model : model;

			if ( self.get_selected_product( false ) > 0 ) {
				composite.console_log( 'debug:views', '\nUpdating "' + self.get_title() + '" selection title...' );
				view.update_selection_title_task( model );
			}
		},

		/**
		 * Gets the selected product title and appends quantity data.
		 */
		get_updated_selection_title: function( model ) {

			var selection_qty            = parseInt( model.get( 'selected_quantity' ), 10 ),
				selection_title          = self.get_selected_product_title( false ),
				selection_qty_string     = selection_qty > 1 ? wc_composite_params.i18n_qty_string.replace( '%s', selection_qty ) : '',
				selection_title_incl_qty = wc_composite_params.i18n_title_string.replace( '%t', selection_title ).replace( '%q', selection_qty_string ).replace( '%p', '' );

			return selection_title_incl_qty;
		},

		/**
		 * Renders the selected product title and the "Clear selection" button.
		 */
		update_selection_title_task: function( model ) {

			var $title_html = self.$component_summary_content.find( '.composited_product_title_wrapper' ),
				view        = this,
				data        = {
					tag:               ( 'single' === composite.settings.layout || 'progressive' === composite.settings.layout ) && ! self.has_options_style( 'thumbnails' ) ? 'p' : 'h4',
					show_title:        'yes' === $title_html.data( 'show_title' ),
					show_selection_ui: self.is_static() ? false : true,
					show_reset_ui:     ( self.show_placeholder_option() && false === self.maybe_is_optional() ) || self.is_optional() || false === self.is_selected_product_valid(),
					selection_title:   view.get_updated_selection_title( model ),
					selection_data:    model.get_product_data()
				};

			var new_template_html = view.templates.selection_title( data );

			if ( new_template_html !== view.templates.selection_title_html ) {
				view.templates.selection_title_html = new_template_html;
				$title_html.html( new_template_html );
			}

			// Remove clearing button if the loaded product is invalid and the current selection can't be reset.
			if ( 'invalid-product' === self.get_selected_product_type() ) {
				var empty_option = self.$component_options_select.find( 'option[value=""]' );
				if ( empty_option.length === 0 || empty_option.first().prop( 'disabled' ) ) {
					self.$component_summary_content.find( '.clear_component_options' ).remove();
				}
			}
		},

		/**
		 * Blocks the composite form and adds a waiting ui cue in the working element.
		 */
		block: function() {

			composite.block( self.$component_options );
		},

		/**
		 * Unblocks the composite form and removes the waiting ui cue from the working element.
		 */
		unblock: function() {

			self.$component_content.removeClass( 'updating' );
			self.$component_thumbnail_options.find( '.loading' ).removeClass( 'loading' );

			composite.unblock( self.$component_options );
		},

		/**
		 * Collect component option change input.
		 */
		option_changed: function( event ) {

			var view                = event.data.view,
				selected_product_id = $( this ).val();

			view.set_option( selected_product_id );

			return false;
		},

		/**
		 * Update model on changing a component option.
		 */
		set_option: function( option_id ) {

			var view = this;

			// Exit if triggering 'change' for the existing selection.
			if ( self.get_selected_product( false ) === option_id ) {
				return false;
			}

			// Toggle thumbnail/radio selection state.
			if ( self.has_options_style( 'thumbnails' ) ) {
				self.$component_thumbnail_options.find( '.selected' ).removeClass( 'selected' );
				self.$component_thumbnail_options.find( '#component_option_thumbnail_' + option_id ).addClass( 'selected loading' );
			} else if ( self.has_options_style( 'radios' ) ) {
				var $selected = self.$component_radio_button_options.find( '.selected' );
				$selected.removeClass( 'selected' );
				$selected.find( 'input' ).prop( 'checked', false );
				self.$component_options.find( '#component_option_radio_button_' + ( option_id === '' ? '0' : option_id ) ).addClass( 'selected' ).find( 'input' ).prop( 'checked', true );
			}

			if ( option_id !== '' ) {

				// Block composite form + add waiting cues.
				this.block();

				// Add updating class to content.
				self.$component_content.addClass( 'updating' );

				setTimeout( function() {
					// Request product details from model and let the model update itself.
					view.model.update_selection( option_id );
				}, 120 );

			} else {

				// Handle selection resets within the view, but update the model data.
				view.model.update_selection( '' );
			}
		},

		/**
		 * Re-set current option.
		 */
		 selection_data_load_error: function() {

		 	var option_id = self.get_selected_product( false );

		 	self.$component_options_select.val( option_id ).change();

		 	this.unblock();

			// Toggle thumbnail/radio selection state.
			if ( self.has_options_style( 'thumbnails' ) ) {
				self.$component_thumbnail_options.find( '.selected' ).removeClass( 'selected' );
				self.$component_thumbnail_options.find( '#component_option_thumbnail_' + option_id ).addClass( 'selected' );
			} else if ( self.has_options_style( 'radios' ) ) {
				var $selected = self.$component_radio_button_options.find( '.selected' );
				$selected.removeClass( 'selected' );
				$selected.find( 'input' ).prop( 'checked', false );
				self.$component_options.find( '#component_option_radio_button_' + ( option_id === '' ? '0' : option_id ) ).addClass( 'selected' ).find( 'input' ).prop( 'checked', true );
			}

		 	window.alert( wc_composite_params.i18n_selection_request_timeout );
		},

		/**
		 * Render the initial model state.
		 */
		render_default: function() {

			var view             = this,
				selected_product = this.model.get( 'selected_product' );

			this.rendered_product = selected_product;

			if ( selected_product ) {
				view.render_content();
			} else {
				view.reset_content();
			}
		},

		component_selection_changed_handler: function( step ) {

			if ( self.step_id !== step.step_id ) {
				return false;
			}

			if ( self.can_autotransition() ) {
				return false;
			}

			this.maybe_render();
		},

		active_step_transition_end_handler: function() {

			this.maybe_render();
		},

		maybe_render: function() {

			var selected_product = this.model.get( 'selected_product' );

			if ( ! composite.is_initialized ) {
				return false;
			}

			if ( this.rendered_product === selected_product ) {
				return false;
			}

			this.render();
		},

		/**
		 * Update view with new selection details passed by model.
		 */
		render: function() {

			var view             = this,
				selected_product = this.model.get( 'selected_product' );

			this.rendered_product = selected_product;

			composite.console_log( 'debug:views', '\nPreparing "' + self.get_title() + '" selection view...' );

			view.prepare_relocation();

			if ( selected_product ) {

				if ( view.is_relocating() ) {

					if ( view.is_relocating_to_origin() ) {

						composite.console_log( 'debug:animations', 'Starting component content height animation...' );

						// Animate component content height to 0.
						// Then, reset relocation and update content.
						self.$component_content.animate( { 'height': 0 }, { duration: view.get_animation_duration( 'close' ), queue: false, always: function() {

							composite.console_log( 'debug:animations', 'Ended component content height animation.' );

							// Put content back to origin position.
							view.reset_relocated_content();
							// Render content.
							view.render_content();

						} } );

						view.load_height = 0;

					} else {

						var do_illusion_scroll = self.$component_content.offset().top < view.$relocation_reference.offset().top && false === self.$component_content.wc_cp_is_in_viewport( true );

						// Animate component content height to 0 while scrolling as much as its height (if needed).
						// Then, update content.
						if ( do_illusion_scroll ) {

							var illusion_scroll_to     = 0,
								illusion_scroll_offset = view.load_height;

							// Introduce async to hopefully do this between repaints and avoid flicker.
							setTimeout( function() {

								illusion_scroll_to = $wc_cp_window.scrollTop() - Math.round( illusion_scroll_offset );

								setTimeout( function() {

									// Scroll as much as the height offset...
									window.scroll( 0, illusion_scroll_to );

									// while setting height to 0.
									self.$component_content.css( { 'height': 0 } );

									setTimeout( function() {

										// Render content.
										view.render_content();

									}, 10 );

								}, 50 );

							}, 50 );

						} else {

							composite.console_log( 'debug:animations', 'Starting component content height animation...' );

							self.$component_content.animate( { 'height': 0 }, { duration: view.get_animation_duration( 'close' ), queue: false, always: function() {

								composite.console_log( 'debug:animations', 'Ended component content height animation.' );

								// Render content.
								view.render_content();

							} } );
						}

						view.load_height = 0;
					}

				} else {

					// Lock height.
					self.$component_content.css( 'height', view.load_height );

					// Render content.
					view.render_content();
				}

			} else {

				composite.console_log( 'debug:animations', 'Starting component content height animation...' );

				// Animate component content height.
				self.$component_content.animate( { 'height': 0 }, { duration: view.get_animation_duration( 'close' ), queue: false, always: function() {

					composite.console_log( 'debug:animations', 'Ended component content height animation.' );

					// Reset content.
					view.reset_content();

					self.$component_content.css( { 'height': 'auto' } );

				} } );
			}
		},

		/**
		 * Prepare the view for relocation.
		 */
		prepare_relocation: function() {

			var view             = this,
				selected_product = this.model.get( 'selected_product' ),
				can_relocate     = this.can_relocate();

			view.load_height = self.$component_content.get( 0 ).getBoundingClientRect().height;

			if ( typeof view.load_height === 'undefined' ) {
				view.load_height = self.$component_content.outerHeight();
			}

			view.relocating           = false;
			view.relocating_to_origin = false;

			// Save initial location of component_content div.
			if ( can_relocate ) {
				if ( false === view.$relocation_origin ) {
					view.$relocation_origin = $( '<div class="component_content_origin">' );
					self.$component_content.before( view.$relocation_origin );
				}
			}

			// Check if the selection details container needs to be relocated.
			if ( can_relocate && selected_product !== '' && self.is_current() ) {

				var relocation_params = view.get_new_relocation_data();

				if ( relocation_params.relocate ) {
					view.$relocation_reference = relocation_params.reference;
					view.relocating            = relocation_params.relocate;
				}

			} else if ( view.is_relocated() ) {
				view.relocating           = true;
				view.relocating_to_origin = true;
			}

			if ( view.relocating ) {

				// Run 'component_selection_details_relocation_started' action - @see WC_CP_Actions_Dispatcher class.
				composite.actions.do_action( 'component_selection_details_relocation_started', [ self ] );

				self.$component_content.addClass( 'relocating' );
			}
		},

		/**
		 * Execute relocation.
		 */
		maybe_relocate_content: function() {

			var view = this;

			// Relocate content.
			if ( view.is_relocating() ) {

				// If the view is already relocated, then move the existing relocation target/container.
				if ( view.is_relocated() ) {

					view.$relocation_reference.after( view.$relocation_target );

				// Otherwise, create a relocation target/container and move content into it.
				} else {

					view.$relocation_target = $( '<li class="component_option_content_container">' );
					view.$relocation_reference.after( view.$relocation_target );
					self.$component_content.appendTo( view.$relocation_target );

					// Run 'component_selection_details_relocation_container_created' action - @see WC_CP_Actions_Dispatcher class.
					composite.actions.do_action( 'component_selection_details_relocation_container_created', [ self ] );
				}

				view.relocated = true;

				self.$component_content.addClass( 'relocated' );
			}
		},

		/**
		 * Renders new selection details in view.
		 */
		render_content: function() {

			var view    = this,
				content = view.model.get_product_data().product_html;

			composite.console_log( 'debug:views', '\nRendering "' + self.get_title() + '" selection view content...' );

			// Preload content.

			var $preload_summary_content = $( '<div class="preload_content" style="position:absolute; left: -100000px;">' );

			$preload_summary_content.html( content );

			self.$el.append( $preload_summary_content );

			var $images = self.is_current() ? $preload_summary_content.find( 'img' ) : [],
				task    = { complete: function( func ) { func(); } };

			if ( composite.is_finalized ) {

				task = new wc_cp_classes.WC_CP_Async_Task( function() {

					var wait       = false,
						async_task = this;

					if ( $images.length > 0 ) {
						$images.each( function() {

							var $image = $( this );

							if ( $image.is( ':visible' ) && $image.height() === 0 && async_task.get_async_time() < 10000 ) {
								wait = true;
								return false;
							}
						} );
					}

					if ( ! wait ) {
						this.done();
					}

				}, 50 );
			}

			task.complete( function() {

				// Reset scripts/classes before replacing markup.
				self.reset_scripts();

				// Execute any pending relocation.
				view.maybe_relocate_content();

				// Put content in place.
				self.$component_summary_content.addClass( 'populated' );
				self.$component_summary_content.html( content );

				// Remove preload wrapper.
				$preload_summary_content.remove();

				view.rendered_content();

				if ( composite.is_finalized ) {

					setTimeout( function() {

						if ( view.is_relocated() ) {
							self.$component_content.removeClass( 'relocating' );
						}

						// Animate content height.
						view.animate_rendered_content();

					}, 300 );
				}

			} );

		},

		/**
		 * Trigger scripts after updating view with selection content.
		 */
		rendered_content: function() {

			composite.console_log( 'debug:views', '\nInitializing "' + self.get_title() + '" view content scripts...' );
			composite.debug_indent_incr();

			if ( this.model.get( 'selected_product' ) > 0 ) {
				self.init_scripts();
			} else {
				self.init_scripts( false );
			}

			composite.debug_indent_decr();
			composite.console_log( 'debug:views', '\nDone initializing "' + self.get_title() + '" view content scripts.' );

			// Run 'component_selection_details_updated' action - @see WC_CP_Actions_Dispatcher class.
			composite.actions.do_action( 'component_selection_details_updated', [ self ] );
		},

		/**
		 * Similar to a render but called when clearing.
		 */
		reset_content: function() {

			// Reset scripts/classes before emptying markup.
			self.reset_scripts();

			// Reset content.
			self.$component_summary_content.html( '' );
			self.$component_summary_content.removeClass( 'populated' );

			// Remove appended navi.
			if ( self.$el.find( '.composite_navigation.movable' ).length > 0 ) {
				composite.$composite_navigation_movable.addClass( 'hidden' );
			}

			this.reset_relocated_content();
			this.rendered_content();

			self.$component_content.removeClass( 'relocating' );
		},

		/**
		 * Animates updated content height.
		 */
		animate_rendered_content: function() {

			// Measure height.
			var new_height     = self.$component_summary.outerHeight( true ),
				animate_height = false,
				view           = this;

			if ( ( self.is_current() || 'single' === composite.settings.layout ) && ( view.is_relocating() || Math.abs( new_height - this.load_height ) > 1 ) ) {
				animate_height = true;
			} else {
				self.$component_content.css( 'height', 'auto' );
			}

			// Animate component content height and scroll to selected product details.
			if ( animate_height ) {

				composite.console_log( 'debug:animations', 'Starting updated content height animation...' );

				// Animate component content height.
				self.$component_content.animate( { 'height': new_height }, { duration: view.get_animation_duration( 'open' ), queue: false, always: function() {

					composite.console_log( 'debug:animations', 'Ended updated content height animation.' );

					// Reset height.
					self.$component_content.css( { 'height' : 'auto' } );

					// Unblock.
					view.unblock();

					// Run 'component_selection_details_animated' action - @see WC_CP_Actions_Dispatcher class.
					composite.actions.do_action( 'component_selection_details_animated', [ self ] );

				} } );

			} else {

				// Unblock.
				view.unblock();

				// Run 'component_selection_details_animated' action - @see WC_CP_Actions_Dispatcher class.
				composite.actions.do_action( 'component_selection_details_animated', [ self ] );
			}
		},

		/**
		 * Move relocated view back to its original position before reloading component options into the 'Component_Options_View'.
		 */
		component_options_flush_handler: function( response, render_type ) {

			if ( this.is_relocated() && render_type === 'reload' && response.result === 'success' ) {
				this.flushing_component_options = true;

				self.$component_content.hide();

				this.reset_relocated_content();

				this.flushing_component_options = false;
			}
		},

		/**
		 * Move relocated view back to its original position.
		 */
		reset_relocated_content: function() {

			var view = this;

			if ( this.is_relocated() ) {

				// Move content to origin.
				view.$relocation_origin.after( self.$component_content );

				// Run 'component_selection_details_relocation_ended' action - @see WC_CP_Actions_Dispatcher class.
				composite.actions.do_action( 'component_selection_details_relocation_ended', [ self ] );

				// Remove origin and relocation container.
				view.$relocation_origin.remove();
				view.$relocation_target.remove();

				// Reset props.
				view.$relocation_origin    = false;
				view.$relocation_target    = false;
				view.$relocation_reference = false;

				view.relocated  = false;
				view.relocating = false;

				self.$component_content.removeClass( 'relocated' );
			}
		},

		/**
		 * True while initializing product scripts, e.g. the variation/bundle script.
		 */
		is_initializing_view_content: function() {
			return self.initializing_scripts;
		},

		/**
		 * True if the view is allowed to relocate below the thumbnail.
		 */
		can_relocate: function() {

			var can_relocate = false;

			if ( 'paged' === composite.settings.layout && self.append_results() && self.has_options_style( 'thumbnails' ) ) {

				if ( 'off' !== self.get_relocation_mode() ) {

					if ( this.is_relocated() ) {

						can_relocate = true;

					} else if ( 'forced' === self.get_relocation_mode() ) {

						can_relocate = true;

					} else if ( 'adaptive' === self.get_relocation_mode() ) {

						var page  = parseInt( self.component_options_model.get( 'page' ), 10 );

						if ( page > 1 ) {
							can_relocate = true;
						}
					}
				}
			}

			return can_relocate;
		},

		/**
		 * True if the component_content container is relocated below the thumbnail.
		 */
		is_relocated: function() {

			return this.relocated;
		},

		/**
		 * True if the component_content container is being relocated below the thumbnail.
		 */
		is_relocating: function() {

			return this.relocating;
		},

		/**
		 * True if the component_content container is being relocated to its origin.
		 */
		is_relocating_to_origin: function() {

			return this.relocating_to_origin;
		},

		/**
		 * Get new relocation parameters for this view, when allowed. Returns:
		 *
		 * - A thumbnail (list item) to be used as the relocation reference (the relocated content should be right after this element).
		 * - A boolean indicating whether the view should be moved under the reference element.
		 */
		get_new_relocation_data: function() {

			var relocation_needed          = false,
				$relocation_reference      = false,
				$selected_thumbnail        = self.$component_options.find( '.component_option_thumbnail.selected' ).closest( '.component_option_thumbnail_container' ),
				thumbnail_to_column_ratio  = $selected_thumbnail.outerWidth( true ) / self.$component_options.outerWidth(),
				$last_thumbnail_in_row     = ( $selected_thumbnail.hasClass( 'last' ) || thumbnail_to_column_ratio > 0.6 ) ? $selected_thumbnail : $selected_thumbnail.nextAll( '.last' ).first();

			if ( $last_thumbnail_in_row.length > 0 ) {
				$relocation_reference = $last_thumbnail_in_row;
			} else {
				$relocation_reference = self.$component_options.find( '.component_option_thumbnail_container' ).last();
			}

			if ( $relocation_reference.next( '.component_option_content_container' ).length === 0 ) {
				relocation_needed = true;
			}

			return {
				relocate:  relocation_needed,
				reference: $relocation_reference
			};
		}

	} );

	var obj = new View( opts );
	return obj;
};
