/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View associated with the composite add-to-cart button.
 */
this.Composite_Add_To_Cart_Button_View = function( opts ) {

	var View = Backbone.View.extend( {

		render_timer: false,
		is_in_widget: false,
		$el_button:   false,
		$el_qty:      false,

		widget_qty_synced: false,

		initialize: function( options ) {

			var model = this.model;

			this.is_in_widget = options.is_in_widget;
			this.$el_button   = options.$el_button;
			this.$el_qty      = this.$el.find( '.quantity input.qty' );

			/**
			 * Update the view when the validation messages change, or when the stock status of the composite changes.
			 */
			composite.actions.add_action( 'composite_availability_status_changed', this.render, 100, this );
			composite.actions.add_action( 'composite_validation_status_changed', this.render, 100, this );

			/*
			 * Events for non-widgetized view.
			 */
			if ( ! this.is_in_widget ) {
				/**
				 * Button click event handler:
				 *
				 * - Check if any issues exist.
				 * - Activate all fields for posting.
				 * - Set invisible selections to empty.
				 */
				this.$el_button.on( 'click', function( event ) {

					if ( model.get( 'passes_validation' ) && model.get( 'is_in_stock' ) ) {

						$.each( composite.get_steps(), function( index, step ) {

							step.$el.find( 'select, input' ).each( function() {
								$( this ).prop( 'disabled', false );
							} );

							if ( step.is_component() ) {

								var has_addons = step.has_addons();

								if ( ! has_addons ) {

									if ( 'bundle' === step.get_selected_product_type() ) {

										var bundle = step.get_bundle_script( step.component_id );

										if ( bundle ) {

											$.each( bundle.bundled_items, function( index, bundled_item ) {

												if ( bundled_item.has_addons() ) {
													has_addons = true;
													return false;
												}

											} );
										}
									}
								}

								if ( has_addons ) {
									step.$component_summary_content.find( '.wc-pao-required-addon [required]' ).prop( 'required', false );
								}

								if ( false === step.step_visibility_model.get( 'is_visible' ) ) {
									step.$component_summary_content.append( '<input name="wccp_component_selection_nil[' + step.step_id + ']" value="1"/>' );
								}
							}
						} );

						$( this ).after( '<input type="hidden" class="active_scenarios" name="wccp_active_scenarios" value="' + composite.scenarios.get_active_scenarios().join( ',' ) + '"/>' );

					} else {
						event.preventDefault();
						window.alert( wc_composite_params.i18n_validation_issues );
					}
				} );
			}

			/*
			 * Events for widgetized view.
			 */
			if ( this.is_in_widget ) {
				/**
				 * Button click event handler: Trigger click in non-widgetized view, located within form.
				 */
				this.$el_button.on( 'click', function() {
					composite.composite_add_to_cart_button_view.$el_button.trigger( 'click' );
				} );

				if ( this.$el_qty.length > 0 ) {

					/**
					 * Copy changed quantity quantity into non-widgetized view.
					 */
					this.$el_qty.on( 'change', { view: this }, function( event ) {

						var view = event.data.view;

						if ( ! view.widget_qty_synced ) {
							composite.console_log( 'debug:views', '\nCopying widget #' + view.is_in_widget + ' quantity value into composite add-to-cart quantity field...' );
							view.widget_qty_synced = true;
							composite.composite_add_to_cart_button_view.$el_qty.val( view.$el_qty.val() ).change();
							view.widget_qty_synced = false;
						}
					} );

					/**
					 * Copy changed composite quantity into view.
					 */
					composite.composite_add_to_cart_button_view.$el_qty.on( 'change', { view: this }, function( event ) {

						var view = event.data.view;

						composite.console_log( 'debug:views', '\nCopying composite add-to-cart quantity value into widget #' + view.is_in_widget + ' quantity field...' );
						view.$el_qty.val( composite.composite_add_to_cart_button_view.$el_qty.val() ).change();
					} );
				}
			}
		},

		render: function() {

			var view  = this,
				model = this.model;

			composite.console_log( 'debug:views', '\nScheduled update of composite add-to-cart button view' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '.' );

			clearTimeout( view.render_timer );
			view.render_timer = setTimeout( function() {
				view.render_task( model );
			}, 10 );
		},

		render_task: function( model ) {

			composite.console_log( 'debug:views', '\nUpdating composite add-to-cart button view' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );

			if ( model.get( 'validation_messages' ).length === 0 && model.get( 'is_in_stock' ) ) {

				if ( composite.settings.button_behaviour === 'new' ) {
					this.$el_button.removeClass( 'disabled' );
				} else {
					this.$el.slideDown( 200 );
				}

			} else {
				if ( composite.settings.button_behaviour === 'new' ) {
					this.$el_button.addClass( 'disabled' );
				} else {
					this.$el.slideUp( 200 );
				}
			}
		}

	} );

	var obj = new View( opts );
	return obj;
};
