/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View associated with the availability status.
 */
this.Composite_Availability_View = function( opts ) {

	var View = Backbone.View.extend( {

		$composite_stock_status: false,
		is_in_widget:            false,
		render_timer:            false,

		initialize: function( options ) {

			this.is_in_widget = options.is_in_widget;

			// Save composite stock status.
			if ( composite.$composite_data.find( '.composite_wrap p.stock' ).length > 0 ) {
				this.$composite_stock_status = composite.$composite_data.find( '.composite_wrap p.stock' ).clone();
			}

			/**
			 * Update the view when the stock statuses change.
			 */
			composite.actions.add_action( 'composite_availability_message_changed', this.render, 100, this );
		},

		render: function() {

			var view  = this,
				model = this.model;

			composite.console_log( 'debug:views', '\nScheduled update of composite availability view' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '.' );

			clearTimeout( view.render_timer );
			view.render_timer = setTimeout( function() {
				view.render_task( model );
			}, 10 );
		},

		render_task: function( model ) {

			composite.console_log( 'debug:views', '\nUpdating composite availability view' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );

			/*
			 * Update composite availability string.
			 */
			var insufficient_stock_components = this.get_insufficient_stock_components();

			if ( insufficient_stock_components.length > 0 ) {
				var composite_out_of_stock_string = wc_composite_params.i18n_insufficient_stock.replace( '%s', wc_cp_join( _.map( insufficient_stock_components, function( component_id ) { return composite.api.get_step_title( component_id ); } ) ) );
				this.$el.html( composite_out_of_stock_string ).slideDown( 200 );
			} else {
				if ( false !== this.$composite_stock_status ) {
					this.$el.html( this.$composite_stock_status ).slideDown( 200 );
				} else {
					this.$el.slideUp( 200 );
				}
			}
		},

		get_insufficient_stock_components: function() {

			var data = [];

			$.each( composite.get_components(), function( index, component ) {
				if ( ! component.step_validation_model.get( 'is_in_stock' ) ) {
					data.push( component.component_id );
				}
			} );

			return data;
		}

	} );

	var obj = new View( opts );
	return obj;
};
