/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View associated with navigation view elements.
 */
this.Composite_Navigation_View = function( opts ) {

	var View = Backbone.View.extend( {

		render_timer:          false,
		render_movable_timer:  false,

		updated_buttons_data:  {},

		navi_in_step:          false,

		template:              false,

		$el_progressive:       composite.$composite_form.find( '.composite_navigation.progressive' ),
		$el_paged_top:         composite.$composite_navigation_top,
		$el_paged_bottom:      composite.$composite_navigation_bottom,
		$el_paged_movable:     composite.$composite_navigation_movable,

		initialize: function() {

			this.template = wp.template( 'wc_cp_composite_navigation' );

			/**
		 	 * Update navigation view elements when a new selection is made.
			 */
			composite.actions.add_action( 'component_selection_changed', this.selection_changed_handler, 110, this );

			/**
			 * Render movable navigation when the product selection view content is updated.
			 */
			composite.actions.add_action( 'component_selection_details_updated', this.selection_details_updated_handler, 10, this );

			/**
		 	 * Update navigation view elements when the contents of an existing selection are changed.
			 */
			composite.actions.add_action( 'component_selection_content_changed', this.selection_content_changed_handler, 100, this );

			/**
		 	 * Update navigation view elements on transitioning to a new step.
			 */
			composite.actions.add_action( 'active_step_transition_start', this.active_step_transition_start_handler, 110, this );

			/**
			 * Update movable navi visibility when appending more options.
			 */
			composite.actions.add_action( 'component_options_state_rendered', this.options_state_rendered_handler, 20, this );

			/**
			 * On clicking the Previous/Next navigation buttons.
			 */
			this.$el.on( 'click', '.page_button', this.clicked_navigation_button );
		},

		/**
		 * Render movable navigation when the product selection view content is updated in the current component.
		 */
		selection_details_updated_handler: function( component ) {
			if ( component.is_current() ) {
				this.render_movable_task();
			}
		},

		/**
		 * Update navigation view elements when the contents of an existing selection are changed.
		 */
		selection_content_changed_handler: function() {

			if ( ! composite.is_initialized ) {
				return false;
			}

			this.render_change();
		},

		/**
	 	 * Updates navigation view elements when a new selection is made.
	 	 * Handled by the composite actions dispatcher.
		 */
		selection_changed_handler: function( step ) {

			if ( ! composite.is_initialized ) {
				return false;
			}

			// Autotransition to next.
			if ( step.can_autotransition() ) {
				composite.show_next_step();
				return false;
			}

			this.render_change();
		},

		/**
	 	 * Update navigation view elements on transitioning to a new step.
		 */
		active_step_transition_start_handler: function() {

			var view = this;

			clearTimeout( view.render_timer );
			view.render( 'transition' );
		},

		/**
		 * Update movable navi visibility in relocated containers when appending more options.
		 */
		options_state_rendered_handler: function( step, changed ) {

			if ( ! composite.is_initialized ) {
				return false;
			}

			if ( step.is_current() && _.contains( changed, 'thumbnails' ) && step.component_selection_view.is_relocated() ) {
				this.render_movable();
			}
		},

		/**
		 * Previous/Next navigation button clicked.
		 */
		clicked_navigation_button: function() {

			var $this = $( this );

			if ( $this.hasClass( 'inactive' ) ) {
				return false;
			}

			if ( composite.has_transition_lock ) {
				return false;
			}

			if ( $this.hasClass( 'next' ) ) {

				if ( composite.get_next_step() ) {
					composite.show_next_step();
				} else {
					composite.composite_viewport_scroller.scroll_viewport( composite.$composite_form.find( '.scroll_final_step' ), { partial: false, duration: 250, queue: false } );
				}

			} else {
				composite.show_previous_step();
			}

			return false;
		},

		update_buttons: function() {

			var view = this,
				data = {
					prev_btn: { btn_classes: '', btn_text: '', btn_link: '', btn_label: '' },
					next_btn: { btn_classes: '', btn_text: '', btn_link: '', btn_label: '' },
				};

			if ( false !== this.updated_buttons_data.button_next_link ) {
				data.next_btn.btn_link = this.updated_buttons_data.button_next_link;
			}

			if ( false !== this.updated_buttons_data.button_prev_link ) {
				data.prev_btn.btn_link = this.updated_buttons_data.button_prev_link;
			}

			if ( false !== this.updated_buttons_data.button_next_html ) {
				data.next_btn.btn_text = this.updated_buttons_data.button_next_html;
			}

			if ( false !== this.updated_buttons_data.button_prev_html ) {
				data.prev_btn.btn_text = this.updated_buttons_data.button_prev_html;
			}

			if ( false !== this.updated_buttons_data.button_next_label ) {
				data.next_btn.btn_label = this.updated_buttons_data.button_next_label;
			}

			if ( false !== this.updated_buttons_data.button_prev_label ) {
				data.prev_btn.btn_label = this.updated_buttons_data.button_prev_label;
			}

			if ( false === this.updated_buttons_data.button_next_visible ) {
				data.next_btn.btn_classes = 'invisible';
			}

			if ( false === this.updated_buttons_data.button_prev_visible ) {
				data.prev_btn.btn_classes = 'invisible';
			}

			if ( false === this.updated_buttons_data.button_next_active ) {
				data.next_btn.btn_classes += ' inactive';
			}

			this.$el.html( view.template( data ) );
		},

		render_change: function() {

			var view = this;

			composite.console_log( 'debug:views', '\nScheduling navigation UI update...' );

			clearTimeout( view.render_timer );
			view.render_timer = setTimeout( function() {
				view.render( 'change' );
			}, 40 );
		},

		render: function( event_type ) {

			composite.console_log( 'debug:views', '\nRendering navigation UI...' );

			var current_step        = composite.get_current_step(),
				next_step           = composite.get_next_step(),
				prev_step           = composite.get_previous_step(),
				view                = this;

			this.updated_buttons_data = {
				button_next_link:    false,
				button_prev_link:    false,
				button_next_label:   false,
				button_prev_label:   false,
				button_next_html:    false,
				button_prev_html:    false,
				button_next_visible: false,
				button_prev_visible: false,
				button_next_active:  false,
			};

			if ( event_type === 'transition' && composite.settings.layout === 'paged' && composite.settings.layout_variation === 'componentized' ) {
				if ( current_step.is_review() ) {
					this.$el_paged_bottom.hide();
				} else {
					this.$el_paged_bottom.show();
				}
			}

			if ( next_step ) {
				this.updated_buttons_data.button_next_link = next_step.get_route();
			}

			if ( prev_step ) {
				this.updated_buttons_data.button_prev_link = prev_step.get_route();
			}

			if ( current_step.is_component() ) {

				// Selectively show next/previous navigation buttons.
				if ( next_step && composite.settings.layout_variation !== 'componentized' ) {

					this.updated_buttons_data.button_next_html    = wc_composite_params.i18n_next_step.replace( '%s', next_step.get_title() );
					this.updated_buttons_data.button_next_label   = next_step.is_review() ? wc_composite_params.i18n_final_step : wc_composite_params.i18n_next_step_label.replace( '%s', next_step.get_title() );
					this.updated_buttons_data.button_next_visible = true;

				} else if ( next_step && composite.settings.layout === 'paged' ) {
					this.updated_buttons_data.button_next_html    = wc_composite_params.i18n_final_step;
					this.updated_buttons_data.button_next_label   = wc_composite_params.i18n_final_step;
					this.updated_buttons_data.button_next_visible = true;
				}
			}

			// Paged previous/next.
			if ( current_step.passes_validation() || ( composite.settings.layout_variation === 'componentized' && current_step.is_component() ) ) {

				if ( next_step ) {
					this.updated_buttons_data.button_next_active = true;
				}

				if ( prev_step && composite.settings.layout === 'paged' && prev_step.is_component() ) {
					this.updated_buttons_data.button_prev_html    = wc_composite_params.i18n_previous_step.replace( '%s', prev_step.get_title() );
					this.updated_buttons_data.button_prev_label   = wc_composite_params.i18n_previous_step_label.replace( '%s', prev_step.get_title() );
					this.updated_buttons_data.button_prev_visible = true;
				} else {
					this.updated_buttons_data.button_prev_html  = '';
					this.updated_buttons_data.button_prev_label = '';
				}

			} else {

				if ( prev_step && prev_step.is_component() ) {

					var product_id = prev_step.get_selected_product();

					if ( product_id > 0 || product_id === '0' || product_id === '' && prev_step.is_optional() ) {

						if ( composite.settings.layout === 'paged' ) {
							this.updated_buttons_data.button_prev_html    = wc_composite_params.i18n_previous_step.replace( '%s', prev_step.get_title() );
							this.updated_buttons_data.button_prev_label   = wc_composite_params.i18n_previous_step_label.replace( '%s', prev_step.get_title() );
							this.updated_buttons_data.button_prev_visible = true;
						}
					}
				}
			}

			/*
			 * Move navigation into the next component when using the progressive layout without toggles.
			 */
			if ( composite.settings.layout === 'progressive' ) {

				var $navi = view.$el_progressive;

				if ( view.navi_in_step !== current_step.step_id ) {

					$navi.slideUp( { duration: 200, always: function() {

						view.update_buttons();
						$navi.appendTo( current_step.$inner_el ).hide();

						view.navi_in_step = current_step.step_id;

						setTimeout( function() {

							var show_navi = false;

							if ( ! current_step.is_last() ) {
								if ( current_step.passes_validation() && ! next_step.has_toggle() ) {
									show_navi = true;
								}
							}

							if ( show_navi ) {
								$navi.slideDown( { duration: 200, queue: false } );
							}

						}, 200 );

					} } );

				} else {

					view.update_buttons();

					var show_navi = false;

					if ( ! current_step.is_last() ) {
						if ( current_step.passes_validation() && ! next_step.has_toggle() ) {
							show_navi = true;
						}
					}

					if ( show_navi ) {
						$navi.slideDown( 200 );
					} else {
						$navi.slideUp( 200 );
					}
				}

			/*
			 * Move navigation when using a paged layout with thumbnails.
			 */
			} else if ( composite.settings.layout === 'paged' ) {

				if ( view.navi_in_step !== current_step.step_id ) {
					current_step.$el.prepend( view.$el_paged_top );
					current_step.$el.append( view.$el_paged_bottom );
					view.navi_in_step = current_step.step_id;
				}

				view.update_buttons();

				if ( event_type === 'transition' ) {
					view.render_movable_task();
				}
			}
		},

		render_movable: function() {

			var view = this;

			composite.console_log( 'debug:views', '\nScheduling movable navigation visibility update...' );

			clearTimeout( view.render_movable_timer );
			view.render_movable_timer = setTimeout( function() {
				view.render_movable_task();
			}, 10 );
		},

		render_movable_task: function() {

			var current_step = composite.get_current_step(),
				view         = this;

			if ( current_step.is_component() && current_step.has_options_style( 'thumbnails' ) ) {

				if ( current_step.get_selected_product() > 0 ) {

					// Measure distance from bottom navi and only append navi in content if far enough.
					var navi_in_content    = current_step.$component_content.find( '.composite_navigation' ).length > 0,
						bottom_navi_nearby = false;

					if ( current_step.append_results() ) {

						if ( current_step.component_selection_view.is_relocated() ) {

							var visible_thumbnails       = current_step.$component_options.find( '.component_option_thumbnail_container' ).not( '.hidden' ),
								selected_thumbnail       = current_step.$component_options.find( '.component_option_thumbnail.selected' ).closest( '.component_option_thumbnail_container' ),
								selected_thumbnail_index = visible_thumbnails.index( selected_thumbnail ) + 1,
								thumbnail_columns        = composite.$composite_form.width() > wc_composite_params.small_width_threshold && false === composite.$composite_form.hasClass( 'legacy_width' ) ? current_step.component_options_view.get_columns() : 1;

							if ( Math.ceil( selected_thumbnail_index / thumbnail_columns ) === Math.ceil( visible_thumbnails.length / thumbnail_columns ) ) {
								bottom_navi_nearby = true;
							}
						}
					}

					if ( ! navi_in_content && ! bottom_navi_nearby ) {
						view.$el_paged_movable.appendTo( current_step.$component_summary );
						navi_in_content = true;
					}

					if ( navi_in_content ) {
						if ( bottom_navi_nearby || current_step.is_static() ) {
							view.$el_paged_movable.addClass( 'hidden' );
						} else {
							view.$el_paged_movable.removeClass( 'hidden' );
						}
					}
				}
			}
		}

	} );

	var obj = new View( opts );
	return obj;
};
