/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View associated with the composite pagination template.
 */
this.Composite_Pagination_View = function( opts ) {

	var View = Backbone.View.extend( {

		template: false,
		template_html: '',

		initialize: function() {

			this.template = wp.template( 'wc_cp_composite_pagination' );

			/**
		 	 * Update view when access to a step changes.
			 */
			composite.actions.add_action( 'step_access_changed', this.step_access_changed_handler, 100, this );

			/**
			 * Update outer element classes when the visibility of a step changes.
			 */
			composite.actions.add_action( 'step_visibility_changed', this.step_visibility_changed_handler, 100, this );

			/**
		 	 * Update view elements on transitioning to a new step.
			 */
			composite.actions.add_action( 'active_step_changed', this.active_step_changed_handler, 100, this );

			/**
			 * On clicking a composite pagination link.
			 */
			this.$el.on( 'click', '.pagination_element a', this.clicked_pagination_element );
		},

		step_visibility_changed_handler: function() {

			this.render();
		},

		step_access_changed_handler: function() {

			this.render();
		},

		active_step_changed_handler: function() {

			this.render();
		},

		/**
		 * Pagination element clicked.
		 */
		clicked_pagination_element: function() {

			$( this ).blur();

			if ( composite.has_transition_lock ) {
				return false;
			}

			if ( $( this ).hasClass( 'inactive' ) ) {
				return false;
			}

			var step_id = $( this ).closest( '.pagination_element' ).data( 'item_id' ),
				step    = composite.get_step( step_id );

			if ( step ) {
				composite.navigate_to_step( step );
			}

			return false;
		},

		/**
		 * Renders all elements state (active/inactive).
		 */
		render: function() {

			var data = [];

			if ( ! composite.is_initialized ) {
				return false;
			}

			composite.console_log( 'debug:views', '\nRendering pagination view elements...' );

			$.each( composite.get_steps(), function( index, step ) {

				if ( step.is_visible() ) {

					var item_data = {
						element_id:          step.step_id,
						element_link:        step.get_route(),
						element_title:       step.get_title(),
						element_class:       '',
						element_state_class: ''
					};

					if ( step.is_current() ) {
						item_data.element_state_class = 'inactive';
						item_data.element_class       = 'pagination_element_current';
					} else if ( step.is_locked() ) {
						item_data.element_state_class = 'inactive';
					}

					data.push( item_data );
				}

			} );

			// Pass through 'composite_pagination_view_data' filter - @see WC_CP_Filters_Manager class.
			data = composite.filters.apply_filters( 'composite_pagination_view_data', [ data ] );

			var new_template_html = this.template( data );

			if ( new_template_html !== this.template_html ) {
				this.template_html = new_template_html;
				this.$el.html( new_template_html );
			} else {
				composite.console_log( 'debug:views', '...skipped!' );
			}
		}

	} );

	var obj = new View( opts );
	return obj;
};
