/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View associated with the price template.
 */
this.Composite_Price_View = function( opts ) {

	var View = Backbone.View.extend( {

		render_timer: false,
		is_in_widget: false,

		suffix: '',

		suffix_contains_price_incl: false,
		suffix_contains_price_excl: false,

		$addons_totals:       false,
		show_addons_totals:   false,
		$custom_price_addons: false,

		initialize: function( options ) {

			this.is_in_widget = options.is_in_widget;

			// Add-ons support.
			if ( ! this.is_in_widget && 'yes' === wc_composite_params.is_pao_installed ) {

				this.$addons_totals = composite.$composite_data.find( '#product-addons-total' );

				if ( this.has_addons() ) {

					// Totals visible?
					if ( 1 == this.$addons_totals.data( 'show-sub-total' ) ) {

						// Ensure addons ajax is not triggered at all, as we calculate tax on the client side.
						this.$addons_totals.data( 'show-sub-total', 0 );
						this.$el.after( this.$addons_totals );
						this.show_addons_totals = true;

						/**
						 * Update the addons totals when the composite totals change.
						 */
						composite.actions.add_action( 'composite_totals_changed', this.update_addons_totals, 100, this );

						/**
						 * Update addons grand totals with correct prices without triggering an ajax call.
						 */
						composite.$composite_data.on( 'updated_addons', { view: this }, this.updated_addons_handler );
					}

					// Custom price addons must be handled separately - see 'get_custom_price_addons_raw_price'.
					this.$custom_price_addons = composite.$composite_data.find( '.addon-custom-price, .wc-pao-addon-custom-price' );

				} else {
					this.$addons_totals = false;
				}
			}

			// Suffix.
			if ( wc_composite_params.price_display_suffix !== '' ) {

				this.suffix = ' <small class="woocommerce-price-suffix">' + wc_composite_params.price_display_suffix + '</small>';

				this.suffix_contains_price_incl = wc_composite_params.price_display_suffix.indexOf( '{price_including_tax}' ) > -1;
				this.suffix_contains_price_excl = wc_composite_params.price_display_suffix.indexOf( '{price_excluding_tax}' ) > -1;
			}

			/**
			 * Update the view when the composite totals change.
			 */
			composite.actions.add_action( 'composite_totals_changed', this.render, 100, this );

			/**
			 * Update the view when the validation status changes.
			 */
			composite.actions.add_action( 'composite_validation_status_changed', this.render, 100, this );
		},

		/**
		 * True if the view includes addons.
		 */
		has_addons: function() {
			return 'yes' === wc_composite_params.is_pao_installed && this.$addons_totals && this.$addons_totals.length > 0;
		},

		/**
		 * True if the view includes custom price addons.
		 */
		has_custom_price_addons: function() {
			return this.$custom_price_addons && this.$custom_price_addons.length > 0;
		},

		/**
		 * Populate prices used by the addons script and re-trigger a 'woocommerce-product-addons-update' event.
		 */
		updated_addons_handler: function( event ) {

			var view = event.data.view;

			view.render_task( view.model );
			event.stopPropagation();
		},

		get_addons_raw_price: function() {

			var addons_raw_price = 0;

			if ( ! this.has_addons() ) {
				return addons_raw_price;
			}

			if ( wc_composite_params.pao_version_gte_30 ) {

				var addons     = this.$addons_totals.data( 'price_data' ),
					tax_ratios = composite.data_model.price_data.base_price_tax_ratios;

				$.each( addons, function( i, addon ) {

					if ( addon.is_custom_price ) {

						var tax_ratio_incl = tax_ratios && typeof( tax_ratios.incl ) !== 'undefined' ? Number( tax_ratios.incl ) : false,
							tax_ratio_excl = tax_ratios && typeof( tax_ratios.excl ) !== 'undefined' ? Number( tax_ratios.excl ) : false;

						if ( 'incl' === wc_composite_params.tax_display_shop && 'no' === wc_composite_params.prices_include_tax ) {
							addons_raw_price += addon.cost_raw / ( tax_ratio_incl ? tax_ratio_incl : 1 );
						} else if ( 'excl' === wc_composite_params.tax_display_shop && 'yes' === wc_composite_params.prices_include_tax ) {
							addons_raw_price += addon.cost_raw / ( tax_ratio_excl ? tax_ratio_excl : 1 );
						} else {
							addons_raw_price += addon.cost_raw;
						}

					} else {

						if ( 'quantity_based' === addon.price_type ) {
							addons_raw_price += addon.cost_raw_pu;
						} else if ( 'flat_fee' === addon.price_type ) {
							addons_raw_price += addon.cost_raw;
						} else if ( 'percentage_based' === addon.price_type ) {
							addons_raw_price += addon.cost_raw_pct * composite.data_model.price_data.base_price;
						}
					}

				} );

			} else {

				addons_raw_price  = this.$addons_totals.data( 'addons-raw-price' ) || 0;
				addons_raw_price += this.get_custom_price_addons_raw_price();
			}

			return addons_raw_price;

		},

		get_custom_price_addons_raw_price: function() {

			var model     = this.model,
				raw_price = 0.0;

			if ( this.has_custom_price_addons() ) {

				this.$custom_price_addons.each( function() {

					var $addon          = $( this ),
						raw_addon_price = $addon.val() || 0,
						tax_ratios     = model.price_data.base_price_tax_ratios,
						tax_ratio_incl  = tax_ratios && typeof( tax_ratios.incl ) !== 'undefined' ? Number( tax_ratios.incl ) : false,
						tax_ratio_excl  = tax_ratios && typeof( tax_ratios.excl ) !== 'undefined' ? Number( tax_ratios.excl ) : false;

					raw_addon_price = Math.abs( parseFloat( accounting.unformat( raw_addon_price, wc_composite_params.currency_format_decimal_sep ), 10 ) );

					if ( 'incl' === wc_composite_params.tax_display_shop && 'no' === wc_composite_params.prices_include_tax ) {
						raw_price += raw_addon_price / ( tax_ratio_incl ? tax_ratio_incl : 1 );
					} else if ( 'excl' === wc_composite_params.tax_display_shop && 'yes' === wc_composite_params.prices_include_tax ) {
						raw_price += raw_addon_price / ( tax_ratio_excl ? tax_ratio_excl : 1 );
					} else {
						raw_price += raw_addon_price;
					}

				} );
			}

			return raw_price;
		},

		get_price_html: function( price_data_array ) {

			var model            = this.model,
				price_data       = typeof( price_data_array ) === 'undefined' ? model.price_data : price_data_array,
				composite_totals = typeof( price_data_array ) === 'undefined' ? model.get( 'totals' ) : price_data_array[ 'totals' ],
				total_string     = wc_composite_params.i18n_total ? '<span class="total">' + wc_composite_params.i18n_total + '</span>' : '',
				price_html       = '';

			if ( this.has_addons() ) {

				price_data = $.extend( true, {}, price_data );

				var addons_raw_price = this.get_addons_raw_price();

				if ( addons_raw_price > 0 ) {
					// Recalculate price html with add-ons price embedded in base price.
					price_data.base_price         = Number( price_data.base_price ) + Number( addons_raw_price );
					price_data.base_regular_price = Number( price_data.base_regular_price ) + Number( addons_raw_price );

					price_data       = model.calculate_subtotals( false, price_data, 1 );
					composite_totals = model.calculate_totals( price_data );
				}
			}

			if ( composite_totals.price === 0.0 && price_data[ 'show_free_string' ] === 'yes' ) {

				price_html = wc_composite_params.i18n_price_format.replace( '%t', total_string ).replace( '%p', wc_composite_params.i18n_free ).replace( '%s', '' );

			} else {

				var formatted_price         = wc_cp_price_format( composite_totals.price ),
					formatted_regular_price = wc_cp_price_format( composite_totals.regular_price ),
					formatted_suffix        = this.get_formatted_price_suffix( composite_totals );

				if ( composite_totals.regular_price > composite_totals.price ) {
					formatted_price = wc_composite_params.i18n_strikeout_price_string.replace( '%f', formatted_regular_price ).replace( '%t', formatted_price );
				}

				price_html = wc_composite_params.i18n_price_format.replace( '%t', total_string ).replace( '%p', formatted_price ).replace( '%s', formatted_suffix );
			}

			price_html = '<p class="price">' + price_html + '</p>';

			return composite.filters.apply_filters( 'composite_price_html', [ price_html, this, price_data_array ] );
		},

		/**
		 * Replace totals in price suffix.
		 */
		get_formatted_price_suffix: function( totals ) {

			var model  = this.model,
				suffix = this.suffix;

			totals = typeof( totals ) === 'undefined' ? model.get( 'totals' ) : totals;

			if ( '' !== suffix ) {

				if ( this.suffix_contains_price_incl ) {
					suffix = suffix.replace( '{price_including_tax}', wc_cp_price_format( totals.price_incl_tax ) );
				}

				if ( this.suffix_contains_price_excl ) {
					suffix = suffix.replace( '{price_excluding_tax}', wc_cp_price_format( totals.price_excl_tax ) );
				}
			}

			return suffix;
		},

		render: function() {

			var view  = this,
				model = this.model;

			composite.console_log( 'debug:views', '\nScheduled update of composite price view' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '.' );

			clearTimeout( view.render_timer );
			view.render_timer = setTimeout( function() {
				view.render_task( model );
			}, 10 );
		},

		render_task: function( model ) {

			var show_price   = model.get( 'validation_messages' ).length === 0 && ( model.price_data[ 'total' ] !== model.price_data[ 'base_display_price' ] || 'yes' === model.price_data[ 'has_price_range' ] ),
				update_price = show_price && model.get( 'passes_validation' );

			composite.console_log( 'debug:views', '\nUpdating composite price view' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );

			if ( show_price ) {

				if ( update_price ) {

					var price_html = this.get_price_html();

					this.$el.html( price_html );
					this.$el.removeClass( 'inactive' ).slideDown( 200 );
				}

			} else {
				this.$el.addClass( 'inactive' ).slideUp( 200 );
			}
		}

	} );

	var obj = new View( opts );
	return obj;
};

