/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View that handles the display of simple status messages.
 */
this.Composite_Status_View = function( opts ) {

	var View = Backbone.View.extend( {

		is_active: false,
		template: false,

		worker: false,

		$el_content: false,

		initialize: function( options ) {

			var view = this;

			this.template    = wp.template( 'wc_cp_composite_status' );
			this.$el_content = options.$el_content;

			/**
		 	 * Update the view when its model state changes.
			 */
			this.listenTo( this.model, 'change:status_messages', this.status_changed );

			var Worker = function() {

				var worker = this;

				this.timer = false;
				this.tasks = [];

				this.last_added_task = [];

				this.is_idle = function() {
					return this.timer === false;
				};

				this.work = function() {
					if ( worker.tasks.length > 0 ) {
						var task = worker.tasks.shift();
						view.render( task );
						worker.timer = setTimeout( function() { worker.work(); }, 400 );
					} else {
						clearTimeout( worker.timer );
						worker.timer = false;
					}
				};

				this.add_task = function( messages ) {

					var task = [];

					// Message added...
					if ( _.pluck( _.where( this.last_added_task, { is_old: false } ), 'message_content' ).length < messages.length ) {
						task = _.map( messages, function( message ) { return { message_content: message, is_old: false }; } );
					// Message removed...
					} else {
						task = _.map( _.where( this.last_added_task, { is_old: false } ), function( data ) { return { message_content: data.message_content, is_old: false === _.contains( messages, data.message_content ) }; } );
					}

					this.last_added_task = task;
					this.tasks.push( task );

					if ( _.where( task, { is_old: true } ).length === task.length ) {
						this.tasks.push( [] );
					}
				};

			};

			this.worker = new Worker();
		},

		/**
		 * Renders the status box.
		 */
		render: function( messages ) {

			var view = this;

			if ( messages.length === 0 ) {

				composite.console_log( 'debug:views', '\nHiding composite status view...' );

				this.$el.removeClass( 'visible' );

				setTimeout( function() {
					view.$el.removeClass( 'active' );
				}, 200 );

				this.is_active = false;

			} else {

				composite.console_log( 'debug:views', '\nUpdating composite status view...' );

				this.$el_content.html( this.template( messages ) );

				if ( false === this.is_active ) {

					this.$el.addClass( 'active' );

					setTimeout( function() {
						view.$el.addClass( 'visible' );
					}, 5 );

					this.is_active = true;

				} else {
					setTimeout( function() {
						view.$el.find( '.message:not(.current)' ).addClass( 'old' );
					}, 100 );
				}
			}
		},

		status_changed: function() {

			var	messages = this.model.get( 'status_messages' );

			if ( messages.length > 0 ) {
				this.worker.add_task( _.pluck( messages, 'message_content' ) );
			} else {
				this.worker.add_task( [] );
			}

			if ( this.worker.is_idle() ) {
				this.worker.work();
			}
		}

	} );

	var obj = new View( opts );
	return obj;
};

