/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View associated with the composite summary template.
 */
this.Composite_Summary_View = function( opts ) {

	var View = Backbone.View.extend( {

		update_content_timers: {},
		view_elements:         {},
		is_in_widget:          false,
		template:              false,

		$carousel_wrapper:     false,
		$carousel_buttons:     false,
		$carousel_button_prev: false,
		$carousel_button_next: false,

		carousel_offset:       0,

		initialize: function( options ) {

			var view = this;

			this.template     = wp.template( 'wc_cp_summary_element_content' );
			this.is_in_widget = options.is_in_widget;

			this.$carousel_wrapper = this.$el.parent();

			$.each( composite.get_steps(), function( index, step ) {
				/**
			 	 * Update a single summary view element content when its validation state changes.
				 */
				step.step_validation_model.on( 'change:passes_validation', function() { view.render_element_content( step ); } );
			} );

			$.each( composite.get_components(), function( index, component ) {

				view.view_elements[ component.component_id ] = {

					$summary_element:         view.$el.find( '.summary_element_' + component.component_id ),
					$summary_element_wrapper: view.$el.find( '.summary_element_' + component.component_id + ' .summary_element_wrapper' ),
					$summary_element_inner:   view.$el.find( '.summary_element_' + component.component_id + ' .summary_element_wrapper_inner' ),

					template_html: ''
				};
			} );

			/**
		 	 * Update view when access to a step changes.
			 */
			composite.actions.add_action( 'step_access_changed', this.step_access_changed_handler, 100, this );

			/**
			 * Update outer element classes when the visibility of a step changes.
			 */
			composite.actions.add_action( 'step_visibility_changed', this.step_visibility_changed_handler, 100, this );

			/**
			 * Update a single summary view element content when its quantity changes.
			 */
			composite.actions.add_action( 'component_quantity_changed', this.quantity_changed_handler, 100, this );

			/**
			 * Update a single summary view element content when a new selection is made.
			 */
			composite.actions.add_action( 'component_selection_changed', this.selection_changed_handler, 100, this );

			/**
			 * Update a single summary view element content when the contents of an existing selection change.
			 */
			composite.actions.add_action( 'component_selection_content_changed', this.selection_changed_handler, 100, this );

			/**
			 * Update a single summary view element price when its totals change.
			 */
			composite.actions.add_action( 'component_totals_changed', this.component_totals_changed_handler, 100, this );

			/**
			 * Update all summary view elements on transitioning to a new step.
			 */
			if ( composite.settings.layout !== 'single' ) {
				composite.actions.add_action( 'active_step_transition_start', this.active_step_changed_handler, 100, this );
			}

			/**
			 * On clicking a summary link.
			 */
			this.$el.on( 'click', '.summary_element_select', this.clicked_summary_element );


			if ( this.is_in_widget && this.get_columns() > 1 ) {

				this.$carousel_buttons     = this.$el.closest( '.widget_composite_summary_details_wrapper' ).find( '.summary_carousel_button' );
				this.$carousel_button_prev = this.$carousel_buttons.first();
				this.$carousel_button_next = this.$carousel_buttons.last();

				/**
				 * On clicking a carousel link.
				 */
				this.$carousel_button_prev.on( 'click', { view: this }, this.clicked_carousel_button_prev );
				this.$carousel_button_next.on( 'click', { view: this }, this.clicked_carousel_button_next );

				/**
				 * Move carousel viewport on transitioning to a new step.
				 */
				if ( 'yes' === composite.settings.summary_carousel_autoscroll ) {
					composite.actions.add_action( 'active_step_transition_end', this.sync_carousel_pos, 100, this );
				}

				/**
				 * Recalculate carousel width on resize.
				 */
				$wc_cp_window.resize( function() {

					if ( ! composite.is_initialized ) {
						return false;
					}

					var summary_element_columns = view.get_columns(),
						carousel_viewport_width = view.get_carousel_viewport_width(),
						carousel_elements_count = view.get_summary_element_indexes().length,
						carousel_width          = 0;

					carousel_width = carousel_elements_count > summary_element_columns ? carousel_viewport_width * carousel_elements_count / summary_element_columns : carousel_viewport_width;

					view.$el.css( { width: carousel_width } );
				} );
			}
		},

		step_access_changed_handler: function( step ) {

			this.render_element_state( step );
		},

		step_visibility_changed_handler: function( step ) {

			this.render_element_visibility( step );
			this.render_columns( step.step_index );
		},

		active_step_changed_handler: function() {

			this.render_state();
		},

		selection_changed_handler: function( step ) {

			this.render_element_content( step );
		},

		quantity_changed_handler: function( step ) {

			this.render_element_content( step );
		},

		component_totals_changed_handler: function( step ) {

			this.render_element_content( step );
		},

		/**
		 * Summary element clicked.
		 */
		clicked_summary_element: function() {

			if ( composite.has_transition_lock ) {
				return false;
			}

			if ( $( this ).hasClass( 'disabled' ) ) {
				return false;
			}

			var step_id = $( this ).closest( '.summary_element' ).data( 'item_id' ),
				step    = composite.get_step( step_id );

			if ( step === false ) {
				return false;
			}

			if ( ! step.is_current() || composite.settings.layout === 'single' ) {
				composite.navigate_to_step( step );
			}

			return false;
		},

		clicked_carousel_button_next: function( event ) {

			if ( $( this ).hasClass( 'inactive' ) ) {
				return;
			}

			event.data.view.scroll_carousel( 'incr' );
		},

		clicked_carousel_button_prev: function( event ) {

			if ( $( this ).hasClass( 'inactive' ) ) {
				return;
			}

			event.data.view.scroll_carousel( 'decr' );
		},

		get_columns: function() {

			return parseInt( this.$el.data( 'summary_columns' ), 10 );
		},

		get_carousel_viewport_width: function() {

			var width = null;

			if ( this.get_columns() > 1 && this.is_in_widget ) {

				width = this.$carousel_wrapper.get( 0 ).getBoundingClientRect().width;

				if ( typeof width === 'undefined' ) {
					width = this.$carousel_wrapper.width();
				}
			}

			return width;
		},

		get_summary_element_indexes: function() {

			var elements = [];

			$.each( composite.get_steps(), function( index, step ) {
				if ( step.is_component() && step.is_visible() ) {
					elements.push( step.step_index );
				}
			} );

			return elements;
		},

		/**
		 * Returns a single element's price (scheduler).
		 */
		get_element_price_html: function( step ) {

			var price_data = composite.data_model.price_data,
				price_html = '';

			if ( step.is_component() && step.is_subtotal_visible() && step.passes_validation() ) {

				var component    = step,
					component_id = component.component_id,
					product_id   = component.get_selected_product_type() === 'variable' ? component.get_selected_variation( false ) : component.get_selected_product( false ),
					qty          = component.get_selected_quantity();

				// Update price.
				if ( product_id > 0 && qty > 0 ) {

					var component_totals = composite.data_model.get( 'component_' + component_id + '_totals' );

					if ( price_data[ 'is_priced_individually' ][ component_id ] === 'no' && component_totals.price === 0.0 && component_totals.regular_price === 0.0 ) {

						price_html = '';

					} else {

						var formatted_price         = wc_cp_price_format( component_totals.price ),
							formatted_regular_price = wc_cp_price_format( component_totals.regular_price );

						if ( component_totals.regular_price > component_totals.price ) {
							formatted_price = wc_composite_params.i18n_strikeout_price_string.replace( '%f', formatted_regular_price ).replace( '%t', formatted_price );
						}

						price_html = '<span class="price summary_element_content">' + formatted_price + '</span>';
					}
				}
			}

			return price_html;
		},

		sync_carousel_pos: function( step ) {

			if ( ! step.is_component() ) {
				return;
			}

			var carousel_columns          = this.get_columns(),
				carousel_elements_indexes = this.get_summary_element_indexes(),
				carousel_elements_count   = carousel_elements_indexes.length,
				carousel_viewport_width   = this.get_carousel_viewport_width(),
				carousel_offset           = this.carousel_offset,
				carousel_width            = carousel_elements_count > carousel_columns ? carousel_viewport_width * carousel_elements_count / carousel_columns : carousel_viewport_width,
				carousel_column_width     = 1 / carousel_elements_count * carousel_width,
				current_step_offset       = 0;

			// Find position of viewed step in carousel.
			$.each( carousel_elements_indexes, function( index, step_index ) {
				if ( step_index === step.step_index ) {
					current_step_offset = index * carousel_column_width;
					return false;
				}
			} );

			// If further to the right, scroll the viewport until it's visible.
			if ( current_step_offset > carousel_viewport_width + carousel_offset - 1 ) {
				this.scroll_carousel( current_step_offset - ( carousel_columns - 1 ) * carousel_column_width );
			}

			// If further to the left, scroll the viewport to it.
			if ( current_step_offset < carousel_offset - 1 ) {
				this.scroll_carousel( current_step_offset );
			}
		},

		scroll_carousel: function( scroll_to ) {

			var $this = $( this );

			if ( $this.hasClass( 'inactive' ) ) {
				return;
			}

			if ( 'incr' !== scroll_to && 'decr' !== scroll_to ) {
				scroll_to = parseInt( scroll_to, 10 );
			}

			var carousel_columns        = this.get_columns(),
				carousel_elements_count = this.get_summary_element_indexes().length,
				carousel_viewport_width = this.get_carousel_viewport_width(),
				carousel_offset         = this.carousel_offset,
				carousel_width          = carousel_elements_count > carousel_columns ? carousel_viewport_width * carousel_elements_count / carousel_columns : carousel_viewport_width,
				carousel_column_width   = 1 / carousel_elements_count * carousel_width;

			if ( 'incr' === scroll_to ) {
				carousel_offset += Math.round( carousel_columns * composite.settings.summary_carousel_scroll_coeff ) * carousel_column_width;
			} else if ( 'decr' === scroll_to ) {
				carousel_offset -= Math.round( carousel_columns * composite.settings.summary_carousel_scroll_coeff ) * carousel_column_width;
			} else {
				carousel_offset = scroll_to;
			}

			// Viewport out of bounds?
			if ( carousel_offset + carousel_viewport_width - carousel_width > 1 ) {
				carousel_offset = carousel_width - carousel_viewport_width;
			}

			if ( carousel_offset < 1 ) {
				carousel_offset = 0;
			}

			// Viewport at first element?
			if ( carousel_offset < 1 ) {
				this.$carousel_button_prev.addClass( 'inactive' );
			} else {
				this.$carousel_button_prev.removeClass( 'inactive' );
			}

			// Viewport at last element?
			if ( carousel_offset + carousel_viewport_width + carousel_column_width - carousel_width > 1 ) {
				this.$carousel_button_next.addClass( 'inactive' );
			} else {
				this.$carousel_button_next.removeClass( 'inactive' );
			}

			this.carousel_offset = carousel_offset;

			this.$el.css( { transform: 'translateX(-' + carousel_offset + 'px)' } );
		},

		render_columns: function( after_index ) {

			if ( ! composite.is_initialized ) {
				return false;
			}

			var summary_element_loop    = 0,
				summary_element_columns = this.get_columns(),
				view                    = this;

			composite.console_log( 'debug:views', '\nUpdating summary view element columns...' );

			after_index = typeof( after_index ) === 'undefined' ? 0 : after_index;

			// Rendering a carousel?
			if ( summary_element_columns > 1 && view.is_in_widget ) {

				var carousel_width          = 0,
					carousel_column_width   = 0,
					carousel_elements_count = this.get_summary_element_indexes().length,
					carousel_viewport_width = this.get_carousel_viewport_width();

				after_index    = 0;
				carousel_width = carousel_elements_count > summary_element_columns ? carousel_viewport_width * carousel_elements_count / summary_element_columns : carousel_viewport_width;

				carousel_column_width = 1 / carousel_elements_count * carousel_width;

				if ( carousel_elements_count > summary_element_columns ) {
					view.$carousel_buttons.removeClass( 'disabled' );
				} else {
					view.$carousel_buttons.addClass( 'disabled' );
				}

				// Viewport out of bounds?
				if ( view.carousel_offset + carousel_viewport_width - carousel_width > 1 ) {
					view.carousel_offset = carousel_width - carousel_viewport_width;
				}

				// Viewport at first element?
				if ( view.carousel_offset < 1 ) {
					view.$carousel_button_prev.addClass( 'inactive' );
				} else {
					view.$carousel_button_prev.removeClass( 'inactive' );
				}

				// Viewport includes last element?
				if ( view.carousel_offset + carousel_viewport_width + carousel_column_width - carousel_width > 1 ) {
					view.$carousel_button_next.addClass( 'inactive' );
				} else {
					view.$carousel_button_next.removeClass( 'inactive' );
				}

				view.$el.css( { width: carousel_width, transform: 'translateX(-' + view.carousel_offset + 'px)' } );
			}

			$.each( composite.get_steps(), function( index, step ) {

				if ( typeof view.view_elements[ step.step_id ] === 'undefined' ) {
					return true;
				}

				if ( step.step_index < after_index ) {
					if ( step.is_visible() ) {
						summary_element_loop++;
					}
					return true;
				}

				var summary_element_classes = '';

				if ( step.is_visible() ) {

					summary_element_loop++;

					if ( ( ( summary_element_loop - 1 ) % summary_element_columns ) == 0 || summary_element_columns == 1 ) {
						summary_element_classes += ' first';
					}

					if ( summary_element_loop % summary_element_columns == 0 ) {
						summary_element_classes += ' last';
					}
				}

				if ( summary_element_columns > 1 && view.is_in_widget ) {
					view.view_elements[ step.step_id ].$summary_element.removeClass( 'first last' );
				} else {
					view.view_elements[ step.step_id ].$summary_element.removeClass( 'first last' ).addClass( summary_element_classes );
				}

				composite.debug_indent_incr();
				view.render_element_content( step );
				composite.debug_indent_decr();
			} );
		},

		/**
		 * Renders all elements visibility.
		 */
		render_visibility: function() {

			if ( ! composite.is_initialized ) {
				return false;
			}

			var view = this;

			composite.console_log( 'debug:views', '\nRendering summary view element visibility' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );
			composite.debug_indent_incr();
			$.each( composite.get_steps(), function( index, step ) {
				view.render_element_visibility( step );
			} );
			composite.debug_indent_decr();
		},

		/**
		 * Renders all elements state (active/inactive).
		 */
		render_state: function() {

			if ( ! composite.is_initialized ) {
				return false;
			}

			var view = this;

			composite.console_log( 'debug:views', '\nRendering summary view element states' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );
			composite.debug_indent_incr();
			$.each( composite.get_steps(), function( index, step ) {
				view.render_element_state( step );
			} );
			composite.debug_indent_decr();
		},

		/**
		 * Render content.
		 */
		render_content: function() {

			var view = this;

			composite.console_log( 'debug:views', '\nRendering summary view element contents' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );
			composite.debug_indent_incr();
			$.each( composite.get_steps(), function( index, step ) {
				view.render_element_content( step );
			} );
			composite.debug_indent_decr();
		},

		/**
		 * Render view.
		 */
		render: function() {

			composite.console_log( 'debug:views', '\nRendering summary view elements' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );
			composite.debug_indent_incr();
			this.render_visibility();
			this.render_state();
			this.render_columns();
			this.render_content();
			composite.debug_indent_decr();
		},

		/**
		 * Renders a single element's content (scheduler).
		 */
		render_element_content: function( step ) {

			if ( ! composite.is_initialized ) {
				return false;
			}

			var view = this;

			if ( typeof this.view_elements[ step.step_id ] === 'undefined' ) {
				return false;
			}

			if ( step.component_selection_model.has_pending_updates() ) {
				return false;
			}

			composite.console_log( 'debug:views', '\nScheduled update of "' + step.get_title() + '" summary view element content' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '.' );

			if ( typeof( this.update_content_timers[ step.step_index ] ) !== 'undefined' ) {
				clearTimeout( view.update_content_timers[ step.step_index ] );
			}

			this.update_content_timers[ step.step_index ] = setTimeout( function() {
				view.render_element_content_task( step );
			}, step.is_component() && '' === step.get_selected_product() ? 200 : 10 );
		},

		/**
		 * Renders a single element's content.
		 */
		render_element_content_task: function( step ) {

			composite.console_log( 'debug:views', '\nRendering "' + step.get_title() + '" summary view element content' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );

			if ( step.is_component() ) {

				var component           = step,
					component_id        = component.component_id,

					$item_summary_outer = this.view_elements[ component_id ].$summary_element_wrapper,
					$item_summary_inner = this.view_elements[ component_id ].$summary_element_inner,

					template_html       = this.view_elements[ component_id ].template_html,

					content_data        = {
						element_index:           step.get_title_index(),
						element_title:           step.get_title(),
						element_selection_title: '',
						element_action:          '',
						element_label:           '',
						element_button_classes:  '',
						element_image_src:       '',
						element_image_srcset:    '',
						element_image_sizes:     '',
						element_image_title:     '',
						element_price:           ''
					},

					image_data          = false,
					load_height         = 0,
					is_visible          = this.$el.is( ':visible' );

				// Lock height if animating.
				if ( is_visible ) {

					load_height = $item_summary_inner.get( 0 ).getBoundingClientRect().height;

					if ( typeof load_height === 'undefined' ) {
						load_height = $item_summary_inner.outerHeight();
					}

					$item_summary_outer.css( 'height', load_height );
				}

				// Selection title.
				content_data.element_selection_title = component.get_selected_product_title( true, false );
				// Is in widget?
				content_data.element_is_in_widget    = this.is_in_widget;
				// Link.
				content_data.element_button_link     = component.get_route();

				// Action text.
				if ( content_data.element_selection_title && component.passes_validation() ) {
					if ( component.is_static() ) {
						content_data.element_action = wc_composite_params.i18n_summary_static_component;
					} else {
						content_data.element_action = wc_composite_params.i18n_summary_configured_component;
					}
				} else {
					content_data.element_action = wc_composite_params.i18n_summary_empty_component;
				}

				content_data.element_label = wc_composite_params.i18n_summary_action_label.replace( '%a', content_data.element_action ).replace( '%c', content_data.element_title );

				// Selection image data.
				image_data = component.get_selected_product_image_data( false );

				if ( false === image_data ) {
					image_data = component.get_placeholder_image_data();
				}

				if ( image_data ) {
					content_data.element_image_src    = image_data.image_src;
					content_data.element_image_srcset = image_data.image_srcset ? image_data.image_srcset : '';
					content_data.element_image_sizes  = image_data.image_sizes ? image_data.image_sizes : '';
					content_data.element_image_title  = image_data.image_title;
				}

				// Selection price.
				content_data.element_price = this.get_element_price_html( step );

				// Pass through 'component_summary_element_content_data' filter - @see WC_CP_Filters_Manager class.
				content_data = composite.filters.apply_filters( 'component_summary_element_content_data', [ content_data, component, this ] );

				var new_template_html = this.template( content_data );

				if ( new_template_html !== template_html ) {

					this.view_elements[ component_id ].template_html = new_template_html;

					// Update content.
					$item_summary_inner.html( new_template_html );

				} else {
					composite.console_log( 'debug:views', '...skipped!' );
				}

				// Update element class.
				if ( component.passes_validation() ) {
					$item_summary_outer.addClass( 'configured' );
				} else {
					$item_summary_outer.removeClass( 'configured' );
				}

				// Run 'component_summary_content_updated' action to allow 3rd party code to add data to the summary - @see WC_CP_Actions_Dispatcher class.
				composite.actions.do_action( 'component_summary_content_updated', [ component, this ] );

				if ( is_visible ) {

					// Preload images before animating.
					var $image = $item_summary_inner.find( '.summary_element_image img' ),
						task   = new wc_cp_classes.WC_CP_Async_Task( function() {

						var wait       = false,
							async_task = this;

						if ( image_data.image_src && $image.is( ':visible' ) ) {

							if ( $image.height() === 0 && async_task.get_async_time() < 1000 ) {
								wait = true;
								return false;
							}
						}

						if ( ! wait ) {
							this.done();
						}

					}, 50 );

					// Animate.
					task.complete( function() {

						// Measure height.
						var new_height     = $item_summary_inner.outerHeight( true ),
							animate_height = false;

						if ( Math.abs( new_height - load_height ) > 1 ) {
							animate_height = true;
						} else {
							$item_summary_outer.css( 'height', 'auto' );
						}

						if ( animate_height ) {

							$item_summary_outer.animate( { 'height': new_height }, {

								duration: 200,
								queue:    true,
								start:    function() {

									composite.console_log( 'debug:animations', 'Starting updated summary element content animation...' );

								},
								always:   function() {

									composite.console_log( 'debug:animations', 'Ended updated summary element content animation.' );

									$item_summary_outer.css( { 'height': 'auto' } );

								}

							} );
						}

					} );
				}
			}
		},

		/**
		 * Renders a single element's state (active/inactive).
		 */
		render_element_visibility: function( step ) {

			if ( ! composite.is_initialized ) {
				return false;
			}

			if ( typeof this.view_elements[ step.step_id ] === 'undefined' ) {
				return false;
			}

			var $element = this.view_elements[ step.step_id ].$summary_element;

			composite.console_log( 'debug:views', '\nUpdating "' + step.get_title() + '" summary view element visibility' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );

			if ( false === step.is_visible() ) {
				$element.addClass( 'hidden' );
			} else {
				$element.removeClass( 'hidden' );
			}
		},

		/**
		 * Renders a single element's state (active/inactive).
		 */
		render_element_state: function( step ) {

			if ( ! composite.is_initialized ) {
				return false;
			}

			if ( typeof this.view_elements[ step.step_id ] === 'undefined' ) {
				return false;
			}

			var $element_wrapper = this.view_elements[ step.step_id ].$summary_element_wrapper;

			composite.console_log( 'debug:views', '\nUpdating "' + step.get_title() + '" summary view element state' + ( this.is_in_widget ? ' (widget #' + this.is_in_widget + ')' : '' ) + '...' );

			if ( step.is_current() ) {

				$element_wrapper.removeClass( 'disabled' );

				if ( composite.settings.layout !== 'single' ) {
					$element_wrapper.addClass( 'selected' );
				}

			} else {

				if ( step.is_locked() ) {

					$element_wrapper.removeClass( 'selected' );
					$element_wrapper.addClass( 'disabled' );

				} else {

					$element_wrapper.removeClass( 'disabled' );
					$element_wrapper.removeClass( 'selected' );
				}
			}
		}

	} );

	var obj = new View( opts );
	return obj;
};
