/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * View associated with the Composite Summary Widget and its elements.
 */
this.Composite_Widget_View = function( opts ) {

	var View = Backbone.View.extend( {

		show_hide_timer: false,

		initialize: function( options ) {

			this.$el.removeClass( 'cp-no-js' );

			this.validation_view = new composite.view_classes.Composite_Validation_View( {
				is_in_widget: options.widget_count,
				el:           this.$el.find( '.widget_composite_summary_error .composite_message' ),
				model:        composite.data_model,
			} );

			this.price_view = new composite.view_classes.Composite_Price_View( {
				is_in_widget: options.widget_count,
				el:           this.$el.find( '.widget_composite_summary_price .composite_price' ),
				model:        composite.data_model,
			} );

			this.availability_view = new composite.view_classes.Composite_Availability_View( {
				is_in_widget: options.widget_count,
				el:           this.$el.find( '.widget_composite_summary_availability .composite_availability' ),
				model:        composite.data_model,
			} );

			this.add_to_cart_button_view = new composite.view_classes.Composite_Add_To_Cart_Button_View( {
				is_in_widget: options.widget_count,
				el:           this.$el.find( '.widget_composite_summary_button .composite_button' ),
				$el_button:   this.$el.find( '.widget_composite_summary_button .composite_button .composite_add_to_cart_button' ),
				model:        composite.data_model,
			} );

			this.composite_summary_view = new composite.view_classes.Composite_Summary_View( {
				is_in_widget: options.widget_count,
				el:           this.$el.find( '.widget_composite_summary_elements' ),
			} );

			// Run 'widget_view_initialized' action - @see WC_CP_Composite_Dispatcher class.
			composite.actions.do_action( 'widget_view_initialized', [ options, this ] );

			/**
			 * Show/hide the widget when transitioning to a new step.
			 */
			if ( composite.settings.layout === 'paged' ) {
				composite.actions.add_action( 'active_step_changed', this.active_step_changed_handler, 100, this );
			} else {
				this.show_hide();
			}
		},

		active_step_changed_handler: function() {

			this.show_hide();
		},

		show_hide: function() {

			var view = this;

			clearTimeout( view.show_hide_timer );
			this.show_hide_timer = setTimeout( function() {
				view.show_hide_task();
			}, 20 );
		},

		show_hide_task: function() {

			var view           = this,
				delay          = 0,
				is_review      = composite.get_current_step().is_review(),
				is_first_load  = typeof this.$el.data( 'is_hidden' ) === 'undefined',
				show_in_review = typeof( composite.settings.show_widget_in_review_step ) !== 'undefined' && 'yes' === composite.settings.show_widget_in_review_step ? 'yes' : 'no';

			if ( is_review && 'no' === show_in_review ) {

				this.$el.data( 'is_hidden', true );

				composite.console_log( 'debug:animations', 'Starting widget slide-up animation...' );

				if ( this.$el.hasClass( 'widget_position_default' ) ) {
					delay = 250;
					this.$el.addClass( 'summary_widget_inactive' );
				}

				setTimeout( function() {
					view.$el.slideUp( {
						duration: 250,
						always: function() {
							composite.console_log( 'debug:animations', 'Ended widget slide-up animation.' );
							view.$el.addClass( 'summary_widget_hidden' ).show();
						}
					} );
				}, delay );

			} else if ( this.$el.data( 'is_hidden' ) || is_first_load ) {

				this.$el.data( 'is_hidden', false );

				composite.console_log( 'debug:animations', 'Starting widget slide-down animation...' );

				if ( is_first_load ) {

					this.$el.show().removeClass( 'summary_widget_hidden' );

				} else {

					this.$el.hide().removeClass( 'summary_widget_hidden' ).slideDown( {
						duration: 250,
						always: function() {
							composite.console_log( 'debug:animations', 'Ended widget slide-down animation.' );
						}
					} );
				}

				setTimeout( function() {
					view.$el.removeClass( 'summary_widget_inactive' );
				}, 10 );
			}
		}

	} );

	var obj = new View( opts );
	return obj;
};
