/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * Updates step title elements by listening to step model changes.
 */
this.Step_Title_View = function( step, opts ) {

	var self = step;
	var View = Backbone.View.extend( {

		$step_title_index: false,

		initialize: function() {

			this.$step_title_index = self.$step_title.find( '.step_index' );

			if ( step.is_component && self.has_toggle() ) {

				/**
				 * On clicking toggled component titles.
				 */
				this.$el.on( 'click', this.clicked_title_handler );

				if ( composite.settings.layout === 'progressive' ) {

					/**
				 	 * Update view when access to the step changes.
					 */
					composite.actions.add_action( 'step_access_changed', this.step_access_changed_handler, 100, this );

					/**
				 	 * Update view on transitioning to a new step.
					 */
					composite.actions.add_action( 'active_step_changed', this.active_step_changed_handler, 100, this );
				}
			}

			if ( false !== this.$step_title_index ) {
				/**
				 * Update step title indexes.
				 */
				composite.actions.add_action( 'step_visibility_changed', this.step_visibility_changed_handler, 100, this );
			}
		},

		clicked_title_handler: function() {

			if ( ! self.has_toggle() ) {
				return false;
			}

			if ( 'single' === composite.settings.layout ) {
				wc_cp_toggle_element( self.$el, self.$component_inner );
			} else {

				if ( self.is_current() ) {
					if ( 'progressive' === composite.settings.layout ) {
						step.toggle_step( 'open', true );
					} else {
						return false;
					}
				}

				if ( $( this ).hasClass( 'inactive' ) ) {
					return false;
				}

				composite.navigate_to_step( self );
			}

			return false;
		},

		step_access_changed_handler: function( step ) {

			if ( step.step_id === self.step_id ) {
				this.render_navigation_state();
			}
		},

		active_step_changed_handler: function() {

			this.render_navigation_state();
		},

		/**
		 * Update progressive component title based on lock state.
		 */
		render_navigation_state: function() {

			if ( composite.settings.layout === 'progressive' && self.has_toggle() ) {

				composite.console_log( 'debug:views', '\nUpdating "' + self.get_title() + '" component title state...' );

				if ( self.is_current() ) {
					this.$el.removeClass( 'inactive' );
				} else {
					if ( self.is_locked() ) {
						this.$el.addClass( 'inactive' );
					} else {
						this.$el.removeClass( 'inactive' );
					}
				}
			}
		},

		/**
		 * Render step title index.
		 */
		render_index: function() {

			if ( ! composite.is_initialized ) {
				return false;
			}

			if ( false === this.$step_title_index ) {
				return false;
			}

			// Count number of hidden components before this one.
			var title_index = step.get_title_index();

			// Refresh index in step title.
			this.$step_title_index.text( title_index );
		},

		step_visibility_changed_handler: function( step ) {

			if ( self.step_index < step.step_index ) {
				return false;
			}

			this.render_index();
		}

	} );

	var obj = new View( opts );
	return obj;
};
