/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * Handles the display of step validation messages.
 */
this.Step_Validation_View = function( step, opts ) {

	var self = step;
	var View = Backbone.View.extend( {

		render_timer: false,
		render_html:  false,
		template:     false,

		event_type: '',

		initialize: function() {

			var view      = this;
			this.template = wp.template( 'wc_cp_validation_message' );

			this.listenTo( this.model, 'change:component_messages', function() {

				if ( ! self.is_current() || typeof( self.$component_message ) === 'undefined' ) {
					return false;
				}

				composite.console_log( 'debug:views', '\nScheduling "' + self.get_title() + '" validation message update...' );

				clearTimeout( view.render_timer );
				view.render_timer = setTimeout( function() {

					view.prepare_render( 'change' );

					if ( self.can_autotransition() ) {
						return false;
					}

					view.render();

				}, 10 );
			} );

			/**
			 * Prepare display of component messages when transitioning to this step.
			 */
			if ( composite.settings.layout !== 'single' ) {
				composite.actions.add_action( 'active_step_changed_' + self.step_id, this.active_step_changed_handler, 100, this );
			}

			/**
			 * Display component messages after transitioning to this step.
			 */
			if ( composite.settings.layout !== 'single' ) {
				composite.actions.add_action( 'active_step_transition_end_' + self.step_id, this.active_step_transition_end_handler, 100, this );
			}

			/**
			 * Hide top message out of view.
			 */
			composite.actions.add_action( 'component_selection_details_relocation_started', this.relocation_started_handler, 100, this );

			/**
			 * Move top message to original location.
			 */
			composite.actions.add_action( 'component_selection_details_relocation_ended', this.relocation_ended_handler, 100, this );

			/**
			 * Move top message into new relocation container.
			 */
			composite.actions.add_action( 'component_selection_details_relocation_container_created', this.relocation_container_created_handler, 100, this );
		},

		/**
		 * Hide top message out of view.
		 */
		relocation_started_handler: function( step ) {

			if ( step.step_id === self.step_id ) {

				var $component_message_top = self.$component_message.filter( '.top' );

				composite.console_log( 'debug:views', '\nHiding "' + self.get_title() + '" validation message to prepare for component details relocation...' );

				if ( $component_message_top ) {

					var done = composite.composite_viewport_scroller.illusion_scroll( {
						target: $component_message_top,
						type:   'hide'
					} );

					if ( false === done ) {
						$component_message_top.slideUp( 200 );
					}
				}
			}
		},

		/**
		 * Move top message into new relocation container.
		 */
		relocation_container_created_handler: function( step ) {

			if ( step.step_id === self.step_id ) {

				composite.console_log( 'debug:views', '\nMoving "' + self.get_title() + '" validation message into relocation target...' );

				var $component_message_top = self.$component_message.filter( '.top' );

				if ( $component_message_top ) {
					self.component_selection_view.$relocation_target.prepend( $component_message_top );
				}
			}
		},

		/**
		 * Move top message to original location.
		 */
		relocation_ended_handler: function( step ) {

			if ( step.step_id === self.step_id ) {

				var $component_message_top = self.$component_message.filter( '.top' );

				composite.console_log( 'debug:views', '\nMoving "' + self.get_title() + '" validation message back to its original position...' );

				if ( $component_message_top ) {
					self.component_selection_view.$relocation_origin.after( $component_message_top );
				}
			}
		},

		/**
		 * Shows component messages when transitioning to this step.
		 */
		active_step_changed_handler: function() {

			if ( ! self.is_current() || typeof( self.$component_message ) === 'undefined' ) {
				return false;
			}

			this.prepare_render( 'transition' );

			if ( false === this.render_html ) {
				this.render();
			}
		},

		/**
		 * Shows component messages when transitioning to this step.
		 */
		active_step_transition_end_handler: function() {

			if ( ! self.is_current() || typeof( self.$component_message ) === 'undefined' ) {
				return false;
			}

			if ( false !== this.render_html ) {
				clearTimeout( this.render_timer );
				this.render();
			}
		},

		/**
		 * Prepares validation messages for rendering.
		 */
		prepare_render: function( event_type ) {

			this.event_type = '' === this.event_type ? event_type : this.event_type;

			var display_message;

			composite.console_log( 'debug:views', '\nPreparing "' + self.get_title() + '" validation message update...' );

			this.render_html = false;

			if ( self.passes_validation() || ( composite.settings.layout_variation === 'componentized' && self.is_component() ) ) {
				display_message = false;
			} else {
				display_message = true;
			}

			if ( display_message ) {

				// Don't show the prompt if it's the last component of the progressive layout.
				if ( ! self.is_last() || ! composite.settings.layout === 'progressive' ) {

					// We actually have something to display here.
					var validation_messages = self.get_validation_messages();

					if ( validation_messages.length > 0 ) {
						this.render_html = this.template( validation_messages );
					}
				}
			}

			if ( this.event_type === 'transition' && false === this.render_html ) {
				if ( composite.settings.layout === 'progressive' ) {
					if ( self.has_toggle() ) {
						self.$component_message.hide();
					}
				} else if ( composite.settings.layout === 'paged' ) {
					self.$component_message.hide();
				}
			}
		},

		/**
		 * Renders validation messages.
		 */
		render: function() {

			var view = this;

			composite.console_log( 'debug:views', '\nUpdating "' + self.get_title() + '" validation message...' );

			if ( composite.settings.layout === 'progressive' ) {

				var delay = this.event_type === 'transition' ? 200 : 1;

				setTimeout( function() {

					if ( false !== view.render_html ) {
						self.$component_message.html( view.render_html );
					}

					if ( false === view.render_html ) {
						self.$component_message.slideUp( 200 );
					} else {
						self.$component_message.slideDown( 200 );
					}

				}, delay );

			} else if ( composite.settings.layout === 'paged' ) {

				self.$component_message.each( function( index, el ) {

					var done   = false,
						$el    = $( el ),
						is_top = $el.hasClass( 'top' );

					var delay = is_top && 'change' === view.event_type && self.component_selection_view.is_relocated() ? self.component_selection_view.get_animation_duration( 'close' ) + 50 : 0;

					setTimeout( function() {

						if ( false !== view.render_html ) {
							self.$component_message.html( view.render_html );
						}

						if ( is_top ) {
							done = composite.composite_viewport_scroller.illusion_scroll( {
								target: $el,
								type:   false === view.render_html ? 'hide' : 'show'
							} );
						}

						if ( false === done ) {
							if ( false === view.render_html ) {
								$el.slideUp( 200 );
							} else {
								$el.slideDown( 200 );
							}
						}

					}, delay );

				} );
			}

			this.event_type = '';
		}

	} );

	var obj = new View( opts );
	return obj;
};
