/* @exclude */
/* jshint -W069 */
/* jshint -W041 */
/* jshint -W018 */
/* global composite */
/* @endexclude */

/**
 * Controls viewport auto-scrolling.
 */
this.Composite_Viewport_Scroller = function( opts ) {

	var View = Backbone.View.extend( {

		initialize: function() {

			if ( 'single' === composite.settings.layout ) {

				// Viewport auto-scrolling on the 'show_step' action.
				composite.actions.add_action( 'show_step', this.autoscroll_single, 10, this );

			} else if ( 'paged' === composite.settings.layout ) {

				// Viewport auto-scrolling on the 'active_step_changed' action.
				composite.actions.add_action( 'active_step_changed', this.autoscroll_paged, 120, this );

				// Viewport auto-scrolling on the 'active_step_transition_end' action.
				composite.actions.add_action( 'active_step_transition_end', this.autoscroll_paged_relocated, 10, this );

			} else if ( 'progressive' === composite.settings.layout ) {

				// Viewport auto-scrolling on the 'active_step_transition_end' hook.
				composite.actions.add_action( 'active_step_transition_end', this.autoscroll_progressive, 10, this );
			}

			// Viewport auto-scrolling on the 'component_options_update_requested' hook.
			composite.actions.add_action( 'component_options_update_requested', this.component_options_update_requested, 10, this );

			// Viewport auto-scrolling on the 'component_selection_details_updated' hook.
			composite.actions.add_action( 'component_selection_details_updated', this.selection_details_updated, 10, this );

			// Viewport auto-scrolling on the 'component_selection_details_animated' hook.
			composite.actions.add_action( 'component_selection_details_animated', this.selection_details_animated, 10, this );

			// Viewport auto-scrolling on the 'component_selection_details_relocation_ended' hook.
			composite.actions.add_action( 'component_selection_details_relocation_ended', this.selection_details_relocation_ended, 10, this );
		},

		// Viewport auto-scrolling on the 'component_options_update_requested' hook.
		component_options_update_requested: function( component, request_params, request_type, is_background_request ) {

			if ( 'reload' === request_type && false === is_background_request ) {
				composite.composite_viewport_scroller.scroll_viewport( component.$component_pagination.filter( '.top' ), {
					offset:   50,
					duration: 200,
					partial:  true,
					queue:    false,
					on_complete: function() {
						if ( composite.is_finalized && 'yes' === wc_composite_params.accessible_focus_enabled ) {
							component.$component_pagination.filter( '.top' ).find( '.woocommerce-result-count' ).focus();
						}
					}
				} );
			}
		},

		// Viewport auto-scrolling on the 'component_selection_details_relocation_ended' hook.
		selection_details_relocation_ended: function( component ) {

			if ( component.component_selection_view.flushing_component_options ) {
				setTimeout( function() {
					component.$component_content.slideDown( 250 );
					// Scroll to component options.
					composite.composite_viewport_scroller.scroll_viewport( 'relative', {
						offset:   -component.$component_summary.outerHeight( true ),
						timeout:  0,
						duration: 250,
						queue:    false
					} );
				}, 200 );
			}
		},

		/**
		 * Viewport auto-scrolling on the 'component_selection_details_updated' hook.
		 */
		selection_details_updated: function( component ) {
			if ( composite.is_finalized && ! component.can_autotransition() && component.is_current() ) {
				this.autoscroll_selection_details( component, 'updated' );
			}
		},

		/**
		 * Viewport auto-scrolling on the 'component_selection_details_animated' hook.
		 */
		selection_details_animated: function( component ) {
			if ( ! component.can_autotransition() ) {
				this.autoscroll_selection_details( component, 'animated' );
			}
		},

		autoscroll_selection_details: function( component, action ) {

			var view    = component.component_selection_view,
				partial = true,
				$target = component.$component_content_scroll_target;

			if ( '' === component.get_selected_product( false ) ) {
				return;
			}

			if ( view.is_relocated() ) {
				if ( 'animated' === action ) {
					$target = view.$relocation_target;
					partial = false;
				} else {
					return;
				}
			} else if ( 'animated' === action ) {
				return;
			}

			if ( composite.is_initialized ) {
				composite.composite_viewport_scroller.scroll_viewport( $target, {
					timeout:       50,
					duration:      250,
					queue:         false,
					partial:       partial,
					scroll_method: 'quarter',
					on_complete: function() {

						// Focus and announce change.
						if ( component.has_options_style( 'thumbnails' ) && 'yes' === wc_composite_params.accessible_focus_enabled ) {
							component.$component_summary_content.find( '.composited_product_title_wrapper' ).focus();
						}
					}
				} );
			}
		},

		/**
		 * Single layout auto-scrolling behaviour on the 'show_step' hook - single layout.
		 */
		autoscroll_single: function( step ) {

			var do_scroll = composite.is_initialized;

			// Scroll to the desired section.
			if ( do_scroll ) {
				composite.composite_viewport_scroller.scroll_viewport( step.$el, {
					partial:  false,
					duration: 250,
					queue:    false
				} );
			}
		},

		/**
		 * Paged layout auto-scrolling behaviour on the 'show_step' hook.
		 */
		autoscroll_paged: function( step ) {

			var do_scroll    = composite.is_initialized,
				is_component = step.is_component(),
				component    = is_component ? step : false;

			if ( ! is_component || ! component.component_selection_view.is_relocated() ) {
				if ( do_scroll ) {
					composite.composite_viewport_scroller.scroll_viewport( composite.$composite_transition_helper, {
						timeout:  20,
						partial:  false,
						duration: 250,
						queue:    false
					} );
				}
			}
		},

		/**
		 * Paged layout auto-scrolling behaviour on the 'active_step_transition_end' hook - relocated content.
		 */
		autoscroll_paged_relocated: function( step ) {

			var do_scroll    = composite.is_initialized,
				is_component = step.is_component(),
				component    = is_component ? step : false;

			if ( is_component && component.component_selection_view.is_relocated() ) {
				if ( do_scroll ) {
					composite.composite_viewport_scroller.scroll_viewport( component.$component_content, {
						timeout:       0,
						partial:       false,
						duration:      250,
						queue:         false,
						scroll_method: 'middle'
					} );
				}
			}
		},

		/**
		 * Prog layout auto-scrolling behaviour on the 'active_step_transition_end' hook.
		 */
		autoscroll_progressive: function( step ) {

			var do_scroll = composite.is_initialized;

			// Scroll.
			if ( do_scroll && step.$el.hasClass( 'autoscrolled' ) ) {
				if ( ! step.$step_title.wc_cp_is_in_viewport( false ) ) {
					composite.composite_viewport_scroller.scroll_viewport( step.$el, {
						timeout:  0,
						partial:  false,
						duration: 250,
						queue:    false
					} );
				}
			}
		},

		/**
		 * Scrolls the viewport.
		 */
		scroll_viewport: function( target, params ) {

			var anim_complete;
			var scroll_to;

			var partial         = typeof( params.partial ) === 'undefined' ? true : params.partial;
			var offset          = typeof( params.offset ) === 'undefined' ? 50 : params.offset;
			var timeout         = typeof( params.timeout ) === 'undefined' ? 5 : params.timeout;
			var anim_duration   = typeof( params.duration ) === 'undefined' ? 250 : params.duration;
			var anim_queue      = typeof( params.queue ) === 'undefined' ? false : params.queue;
			var always_complete = typeof( params.always_on_complete ) === 'undefined' ? false : params.always_on_complete;
			var scroll_method   = typeof( params.scroll_method ) === 'undefined' ? false : params.scroll_method;

			var do_scroll       = false;
			var $w              = $wc_cp_window;
			var $d              = $wc_cp_document;

			if ( typeof( params.on_complete ) === 'undefined' || params.on_complete === false ) {
				anim_complete = function() {
					return false;
				};
			} else {
				anim_complete = params.on_complete;
			}

			var scroll_viewport = function() {

				// Scroll viewport by an offset.
				if ( target === 'relative' ) {

					scroll_to = $w.scrollTop() - offset;
					do_scroll = true;

				// Scroll viewport to absolute document position.
				} else if ( target === 'absolute' ) {

					scroll_to = offset;
					do_scroll = true;

				// Scroll to target element.
				} else if ( target.length > 0 && target.is( ':visible' ) && false === target.wc_cp_is_in_viewport( partial ) ) {

					var window_offset = offset;

					if ( scroll_method === 'bottom' || target.hasClass( 'scroll_bottom' ) ) {
						window_offset = $w.height() - target.outerHeight( true ) - offset;
					} else if ( scroll_method === 'middle' ) {
						window_offset = $w.height() / 3 * 2 - target.outerHeight( true ) - offset;
					} else if ( scroll_method === 'quarter' ) {
						window_offset = $w.height() / 4 + offset;
					} else {
						window_offset = parseInt( wc_composite_params.scroll_viewport_top_offset, 10 ) + offset;
					}

					scroll_to = target.offset().top - window_offset;

					// Ensure element top is in viewport.
					if ( target.offset().top < scroll_to ) {
						scroll_to = target.offset().top;
					}

					do_scroll = true;
				}

				if ( do_scroll ) {

					// Prevent out-of-bounds scrolling.
					if ( scroll_to > $d.height() - $w.height() ) {
						scroll_to = $d.height() - $w.height() - 100;
					}

					// Avoid scrolling both html and body.
					var pos            = $wc_cp_html.scrollTop();
					var animate_target = $wc_cp_body;

					$wc_cp_html.scrollTop( $wc_cp_html.scrollTop() - 1 );
					if ( pos != $wc_cp_html.scrollTop() ) {
						animate_target = $wc_cp_html;
					}

					composite.console_log( 'debug:animations', '\nStarting viewport auto-scrolling...' );

					animate_target.animate( { scrollTop: scroll_to }, {
						duration: anim_duration,
						queue:    anim_queue,
						complete: function() {
							composite.console_log( 'debug:animations', '\nEnded viewport auto-scrolling.' );
						},
						always:   anim_complete
					} );

				} else {
					if ( always_complete ) {
						anim_complete();
					}
				}
			};

			if ( timeout > 0 ) {
				setTimeout( function() {
					scroll_viewport();
				}, timeout );
			} else {
				scroll_viewport();
			}
		},

		/**
		 * Hides/shows an element that is above the viewport while keeping the visible viewport area unchanged.
		 */
		illusion_scroll: function( args ) {

			if ( typeof( args ) === 'undefined' || typeof( args[ 'target' ] ) === 'undefined' || ( ! args[ 'target' ] ) ) {
				return null;
			}

			var	$el           = args[ 'target' ],
				scroll_pos    = $wc_cp_window.scrollTop(),
				type          = typeof( args[ 'type' ] ) !== 'undefined' ? args[ 'type' ] : 'hide',
				do_it         = true,
				scroll_to     = 0.0,
				scroll_offset = 0.0;

			if ( 'hide' === type ) {
				if ( 'none' === $el.css( 'display' ) ) {
					return null;
				}
			} else {
				if ( 'none' !== $el.css( 'display' ) ) {
					return null;
				}
			}

			if ( 'hide' !== type ) {
				$el.css( {
					height:  '0',
					display: 'block'
				} );
			}

			do_it = $el.offset().top < scroll_pos && false === $el.wc_cp_is_in_viewport( true );

			if ( do_it ) {

				if ( 'hide' !== type ) {
					$el.css( {
						height:   '',
						position: 'absolute',
					} );
				}

				scroll_offset = $el.get( 0 ).getBoundingClientRect().height;

				if ( typeof scroll_offset === 'undefined' ) {
					scroll_offset = $el.outerHeight();
				}

				do_it = scroll_offset >= 1;
			}

			if ( 'hide' !== type ) {
				$el.css( {
					height:   '',
					position: '',
					display:  'none'
				} );
			}

			if ( do_it ) {

				scroll_to = 'hide' === type ? scroll_pos - Math.round( scroll_offset ) : scroll_pos + Math.round( scroll_offset );

				// Introduce async to hopefully do this between repaints and avoid flicker.
				setTimeout( function() {

					// Scroll as much as the height offset...
					window.scroll( 0, scroll_to );

					if ( 'hide' === type ) {
						// ...while hiding the element.
						$el.hide();
					} else {
						// ...while showing the element.
						$el.show();
					}

				}, 10 );
			}

			return do_it;
		}

	} );

	var obj = new View( opts );
	return obj;
};
